# -*- coding: utf-8 -*-
import os
import shutil
from io import StringIO
from tempfile import mkdtemp

from django.core.management import CommandError, call_command
from django.test import TestCase
from django.test.utils import override_settings

from ...testapp.models import Photo

from unittest.mock import patch


class UnreferencedFilesExceptionsTests(TestCase):
    @override_settings(MEDIA_ROOT=None)
    def test_should_raise_ComandError_if_MEDIA_ROOT_is_not_set(self):
        with self.assertRaisesRegex(
            CommandError, "MEDIA_ROOT is not set, nothing to do"
        ):
            call_command("unreferenced_files")


class UnreferencedFilesTests(TestCase):
    def setUp(self):
        self.media_root_dir = mkdtemp()

    def tearDown(self):
        super().tearDown()
        shutil.rmtree(self.media_root_dir)

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_not_print_any_output(self, m_stdout):
        with override_settings(MEDIA_ROOT=self.media_root_dir):
            call_command("unreferenced_files")

        self.assertIs("", m_stdout.getvalue())

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_unreferenced_hello_txt_file(self, m_stdout):
        fn = os.path.join(self.media_root_dir, "hello.txt")
        open(fn, "a").close()

        with override_settings(MEDIA_ROOT=self.media_root_dir):
            call_command("unreferenced_files")

        self.assertIn(fn, m_stdout.getvalue())

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_not_print_referenced_image_jpg_file(self, m_stdout):
        fn = os.path.join(self.media_root_dir, "image.jpg")
        open(fn, "a").close()

        Photo.objects.create(photo="image.jpg")

        with override_settings(MEDIA_ROOT=self.media_root_dir):
            call_command("unreferenced_files")

        self.assertNotIn(fn, m_stdout.getvalue())



# -*- coding: utf-8 -*-
import sys
from io import StringIO

import django
from django.contrib.auth.models import Permission
from django.core.management import call_command
from django.db import models
from django.test import TestCase


if django.VERSION >= (5, 0):
    DJANGO_EXTENSIONS_NAME = "Django Extensions"
    TESTAPP_NAME = "Testapp"
else:
    DJANGO_EXTENSIONS_NAME = "django_extensions"
    TESTAPP_NAME = "testapp"


class UpdatePermissionsTests(TestCase):
    def setUp(self):
        class PermModel(models.Model):
            class Meta:
                app_label = "django_extensions"
                permissions = (("test_permission", "test_permission"),)

        class TestModel(models.Model):
            class Meta:
                app_label = "testapp"
                permissions = (("testapp_permission", "testapp_permission"),)

    def test_works(self):
        original_stdout = sys.stdout
        out = sys.stdout = StringIO()
        call_command("update_permissions", stdout=out, verbosity=3)
        sys.stdout = original_stdout
        self.assertIn("Can change perm model", out.getvalue())

    def test_should_reload_permission_only_for_specified_apps(self):
        original_stdout = sys.stdout
        out = sys.stdout = StringIO()

        call_command("update_permissions", "--apps=testapp", stdout=out, verbosity=3)

        sys.stdout = original_stdout
        self.assertNotIn(
            f"{DJANGO_EXTENSIONS_NAME} | perm model | Can add perm model",
            out.getvalue(),
        )
        self.assertIn(
            f"{TESTAPP_NAME} | test model | Can add test model", out.getvalue()
        )

    def test_should_reload_permission_only_for_all_apps(self):
        original_stdout = sys.stdout
        out = sys.stdout = StringIO()

        call_command("update_permissions", verbosity=3)

        sys.stdout = original_stdout
        self.assertIn(
            f"{DJANGO_EXTENSIONS_NAME} | perm model | Can add perm model",
            out.getvalue(),
        )
        self.assertIn(
            f"{TESTAPP_NAME} | test model | Can add test model", out.getvalue()
        )

    def test_should_update_permission_if_name_changed(self):
        original_stdout = sys.stdout
        out = sys.stdout = StringIO()

        call_command("update_permissions", verbosity=3, create_only=True)
        self.assertIn(
            f"{TESTAPP_NAME} | test model | testapp_permission", out.getvalue()
        )

        testapp_permission = Permission.objects.get(name="testapp_permission")
        testapp_permission.name = "testapp_permission_wrong"
        testapp_permission.save()

        call_command("update_permissions", verbosity=3, update_only=True)

        sys.stdout = original_stdout
        self.assertIn(
            f"'{TESTAPP_NAME} | test model | testapp_permission_wrong' to '{TESTAPP_NAME} | test model | testapp_permission'",
            out.getvalue(),
        )



# -*- coding: utf-8 -*-
from django.db import models

from django_extensions.db.fields import UniqueFieldMixin


class UniqField(UniqueFieldMixin, models.CharField):
    def __init__(self, *args, **kwargs):
        self.boolean_attr = kwargs.pop("boolean_attr", False)
        self.non_boolean_attr = kwargs.pop("non_boolean_attr", "non_boolean_attr")

        super().__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = (
        "List all permissions for models. "
        "By default, excludes admin, auth, contenttypes, and sessions apps."
    )

    def add_arguments(self, parser):
        parser.add_argument(
            "app_label_model",
            nargs="*",
            help="[app_label.]model(s) to show permissions for.",
        )
        parser.add_argument(
            "--all",
            action="store_true",
            help="Include results for admin, auth, contenttypes, and sessions.",
        )
        parser.add_argument("--app-label", help="App label to dump permissions for.")

    def handle(self, *args, **options):
        app_label_models = options["app_label_model"]
        include_all = options["all"]
        app_label_filter = options["app_label"]

        if include_all:
            content_types = ContentType.objects.order_by("app_label", "model")
        elif app_label_filter:
            content_types = ContentType.objects.filter(
                app_label=app_label_filter.lower()
            ).order_by("app_label", "model")
            if not content_types:
                raise CommandError(
                    f'No content types found for app label "{app_label_filter}".'
                )
        elif not app_label_models:
            excluded = ["admin", "auth", "contenttypes", "sessions"]
            content_types = ContentType.objects.exclude(
                app_label__in=excluded
            ).order_by("app_label", "model")
        else:
            content_types = []
            for value in app_label_models:
                if "." in value:
                    app_label, model = value.split(".")
                    qs = ContentType.objects.filter(app_label=app_label, model=model)
                else:
                    qs = ContentType.objects.filter(model=value)

                if not qs:
                    raise CommandError(f"Content type not found for '{value}'.")
                content_types.extend(qs)

        for ct in content_types:
            self.stdout.write(f"Permissions for {ct}")
            for perm in ct.permission_set.all():
                self.stdout.write(f"    {ct.app_label}.{perm.codename} | {perm.name}")



# -*- coding: utf-8 -*-
import factory
from factory.django import DjangoModelFactory

from .models import Secret


class SecretFactory(DjangoModelFactory):
    """DjangoModelFactory for object Secret."""

    name = factory.Faker("name")
    text = factory.Faker("bs")

    class Meta:
        model = Secret



# -*- coding: utf-8 -*-
import os


def run(*args):
    print("Script called from: %s" % os.getcwd())
    os.chdir(
        os.path.dirname(os.getcwd())
    )  # to test is NONE option freezing start directory



# -*- coding: utf-8 -*-
from django.test import SimpleTestCase
from django_extensions.management.modelviz import generate_graph_data, ON_DELETE_COLORS


class ModelVizTests(SimpleTestCase):
    def test_generate_graph_data_can_render_label(self):
        app_labels = ["auth"]
        data = generate_graph_data(app_labels)

        models = data["graphs"][0]["models"]
        user_data = [x for x in models if x["name"] == "User"][0]
        relation_labels = [x["label"] for x in user_data["relations"]]
        self.assertIn("groups (user)", relation_labels)

    def test_render_unicode_field_label(self):
        app_labels = ["django_extensions"]
        data = generate_graph_data(app_labels, verbose_names=True)
        models = data["graphs"][0]["models"]
        model = [x for x in models if x["name"] == "UnicodeVerboseNameModel"][0]
        fields = dict((_f["name"], _f["label"]) for _f in model["fields"])
        expected = {
            "id": "ID",
            "cafe": "Café",
            "parent_cafe": "Café latte",
        }
        self.assertEqual(expected, fields)

    def test_on_delete_color_coding(self):
        app_labels = ["django_extensions"]
        data = generate_graph_data(app_labels, color_code_deletions=True)

        models = data["graphs"][0]["models"]

        for model in models:
            relations = [
                x
                for x in model["relations"]
                if x["type"] in ("ForeignKey", "OneToOneField")
            ]

            for relation in relations:
                field = [
                    x["field"] for x in model["fields"] if x["name"] == relation["name"]
                ][0]
                on_delete = getattr(field.remote_field, "on_delete", None)
                expected_color = ON_DELETE_COLORS[on_delete]

                self.assertIn("color={}".format(expected_color), relation["arrows"])

    def test_disabled_on_delete_color_coding(self):
        app_labels = ["django_extensions"]
        data = generate_graph_data(app_labels)

        models = data["graphs"][0]["models"]

        for model in models:
            relations = [
                x
                for x in model["relations"]
                if x["type"] in ("ForeignKey", "OneToOneField")
            ]

            for relation in relations:
                self.assertNotIn("color=", relation["arrows"])



# -*- coding: utf-8 -*-
import time
from django.test import TestCase

from .testapp.models import TimestampedTestModel


class ModifiedFieldTest(TestCase):
    def test_update(self):
        t = TimestampedTestModel.objects.create()
        modified = t.modified

        time.sleep(1)

        t.save()
        self.assertNotEqual(modified, t.modified)

    def test_update_no_modified(self):
        t = TimestampedTestModel.objects.create()
        modified = t.modified

        time.sleep(1)

        t.save(update_modified=False)
        self.assertEqual(modified, t.modified)



# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth import get_user_model
from django.db.models import UniqueConstraint
from django.db.models.signals import pre_save

from django_extensions.db.fields import (
    AutoSlugField,
    ModificationDateTimeField,
    RandomCharField,
    ShortUUIDField,
)
from django_extensions.db.fields.json import JSONField
from django_extensions.db.models import ActivatorModel, TimeStampedModel

from .fields import UniqField


class Secret(models.Model):
    name = models.CharField(blank=True, max_length=255, null=True)
    text = models.TextField(blank=True, null=True)

    class Meta:
        app_label = "django_extensions"


class Name(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        app_label = "django_extensions"


class Personality(models.Model):
    description = models.CharField(max_length=50)


class Club(models.Model):
    name = models.CharField(max_length=50)
    created = models.DateTimeField(auto_now=True)


class Note(models.Model):
    note = models.TextField()
    club = models.ForeignKey(Club, null=True, on_delete=models.CASCADE)

    class Meta:
        app_label = "django_extensions"


class Neighborhood(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        app_label = "django_extensions"


class Bank(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        app_label = "django_extensions"


class Person(models.Model):
    name = models.ForeignKey(Name, on_delete=models.CASCADE)
    age = models.PositiveIntegerField()
    children = models.ManyToManyField("self")
    notes = models.ManyToManyField(Note)
    personality = models.OneToOneField(
        Personality,
        null=True,
        on_delete=models.CASCADE,
    )
    clubs = models.ManyToManyField(Club, through="testapp.Membership")
    neighborhood = models.ForeignKey(Neighborhood, on_delete=models.SET_NULL, null=True)
    current_bank = models.ForeignKey(Bank, on_delete=models.PROTECT, null=True)

    class Meta:
        app_label = "django_extensions"


class Membership(models.Model):
    person = models.ForeignKey(Person, on_delete=models.CASCADE)
    club = models.ForeignKey(Club, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now=True)


class Post(ActivatorModel):
    title = models.CharField(max_length=255)

    class Meta:
        app_label = "django_extensions"


class PostWithTitleOrdering(Post):
    class Meta:
        proxy = True
        ordering = ["title"]


class DummyRelationModel(models.Model):
    class Meta:
        app_label = "django_extensions"


class SecondDummyRelationModel(models.Model):
    class Meta:
        app_label = "django_extensions"


class ThirdDummyRelationModel(models.Model):
    class Meta:
        app_label = "django_extensions"


class PostWithUniqFieldCompat(models.Model):
    """
    django-extensions from version 3.0 officially supports only Django 2.2+ which
    introduced Meta.constraints and suggests using it instead of Meta.unique_together
    this is left only to ensure compatibility with Meta.unique_together
    """

    uniq_field = UniqField(
        max_length=255, boolean_attr=True, non_boolean_attr="non_boolean_attr"
    )
    common_field = models.CharField(max_length=10)
    another_common_field = models.CharField(max_length=10)
    many_to_one_field = models.ForeignKey(DummyRelationModel, on_delete=models.CASCADE)
    one_to_one_field = models.OneToOneField(
        SecondDummyRelationModel, on_delete=models.CASCADE
    )
    many_to_many_field = models.ManyToManyField(
        ThirdDummyRelationModel, related_name="posts_with_uniq"
    )

    class Meta:
        app_label = "django_extensions"
        unique_together = (
            "common_field",
            "uniq_field",
        )


class ReverseModelCompat(models.Model):
    post_field = models.ForeignKey(
        PostWithUniqFieldCompat, related_name="reverse_models", on_delete=models.CASCADE
    )

    class Meta:
        app_label = "django_extensions"


class InheritedFromPostWithUniqFieldCompat(PostWithUniqFieldCompat):
    new_field = models.CharField(max_length=10)

    class Meta:
        app_label = "django_extensions"


class PostWithUniqField(models.Model):
    uniq_field = UniqField(
        max_length=255, boolean_attr=True, non_boolean_attr="non_boolean_attr"
    )
    common_field = models.CharField(max_length=10)
    another_common_field = models.CharField(max_length=10)
    many_to_one_field = models.ForeignKey(DummyRelationModel, on_delete=models.CASCADE)
    one_to_one_field = models.OneToOneField(
        SecondDummyRelationModel, on_delete=models.CASCADE
    )
    many_to_many_field = models.ManyToManyField(
        ThirdDummyRelationModel, related_name="posts22_with_uniq"
    )

    class Meta:
        app_label = "django_extensions"
        constraints = [
            models.UniqueConstraint(
                fields=("common_field", "uniq_field"), name="unique_common_uniq_pair"
            ),
            models.CheckConstraint(
                check=~models.Q(common_field=models.F("another_common_field")),
                name="common_and_another_common_differ",
            ),
        ]


class ReverseModel(models.Model):
    post_field = models.ForeignKey(
        PostWithUniqField, related_name="reverse_models", on_delete=models.CASCADE
    )

    class Meta:
        app_label = "django_extensions"


class InheritedFromPostWithUniqField(PostWithUniqField):
    new_field = models.CharField(max_length=10)

    class Meta:
        app_label = "django_extensions"


class AbstractInheritanceTestModelParent(models.Model):
    my_field_that_my_child_will_inherit = models.BooleanField()

    class Meta:
        app_label = "django_extensions"
        abstract = True


class AbstractInheritanceTestModelChild(AbstractInheritanceTestModelParent):
    class Meta:
        app_label = "django_extensions"


class SluggedTestModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title")

    class Meta:
        app_label = "django_extensions"


class SluggedWithConstraintsTestModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title")
    category = models.CharField(max_length=20, null=True)

    class Meta:
        app_label = "django_extensions"
        constraints = [
            UniqueConstraint(
                fields=["slug", "category"],
                name="unique_slug_and_category",
            ),
        ]


class SluggedWithUniqueTogetherTestModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title")
    category = models.CharField(max_length=20, null=True)

    class Meta:
        app_label = "django_extensions"
        unique_together = ["slug", "category"]


class OverridedFindUniqueAutoSlugField(AutoSlugField):
    def find_unique(self, model_instance, field, iterator, *args):
        self.overrided = True
        return super().find_unique(model_instance, field, iterator, *args)


class OverridedFindUniqueModel(models.Model):
    title = models.CharField(max_length=42)
    slug = OverridedFindUniqueAutoSlugField(populate_from="title")


class CustomFuncSluggedTestModel(models.Model):
    def slugify_function(self, content):
        return content.upper()

    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title")

    class Meta:
        app_label = "django_extensions"


class CustomFuncPrecedenceSluggedTestModel(models.Model):
    def custom_slug_one(self, content):
        return content.upper()

    def custom_slug_two(content):
        return content.lower()

    slugify_function = custom_slug_one

    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title", slugify_function=custom_slug_two)

    class Meta:
        app_label = "django_extensions"


class ChildSluggedTestModel(SluggedTestModel):
    class Meta:
        app_label = "django_extensions"


class SluggedTestNoOverwriteOnAddModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="title", overwrite_on_add=False)

    class Meta:
        app_label = "django_extensions"


def get_readable_title(instance):
    return "The title is {}".format(instance.title)


class ModelMethodSluggedTestModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from="get_readable_title")

    class Meta:
        app_label = "django_extensions"

    def get_readable_title(self):
        return get_readable_title(self)


class FunctionSluggedTestModel(models.Model):
    title = models.CharField(max_length=42)
    slug = AutoSlugField(populate_from=get_readable_title)

    class Meta:
        app_label = "django_extensions"


class FKSluggedTestModel(models.Model):
    related_field = models.ForeignKey(SluggedTestModel, on_delete=models.CASCADE)
    slug = AutoSlugField(populate_from="related_field__title")

    class Meta:
        app_label = "django_extensions"


class FKSluggedTestModelCallable(models.Model):
    related_field = models.ForeignKey(
        ModelMethodSluggedTestModel, on_delete=models.CASCADE
    )
    slug = AutoSlugField(populate_from="related_field__get_readable_title")

    class Meta:
        app_label = "django_extensions"


class JSONFieldTestModel(models.Model):
    a = models.IntegerField()
    j_field = JSONField()

    class Meta:
        app_label = "django_extensions"


class ShortUUIDTestModel_field(models.Model):
    a = models.IntegerField()
    uuid_field = ShortUUIDField()

    class Meta:
        app_label = "django_extensions"


class ShortUUIDTestModel_pk(models.Model):
    uuid_field = ShortUUIDField(primary_key=True)

    class Meta:
        app_label = "django_extensions"


class ShortUUIDTestAgregateModel(ShortUUIDTestModel_pk):
    a = models.IntegerField()

    class Meta:
        app_label = "django_extensions"


class ShortUUIDTestManyToManyModel(ShortUUIDTestModel_pk):
    many = models.ManyToManyField(ShortUUIDTestModel_field)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModel(models.Model):
    random_char_field = RandomCharField(length=8, unique=False)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelUnique(models.Model):
    random_char_field = RandomCharField(length=8, unique=True)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelAlphaDigits(models.Model):
    random_char_field = RandomCharField(length=8, unique=True)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelLowercaseAlphaDigits(models.Model):
    random_char_field = RandomCharField(length=8, lowercase=True)

    class Meta:
        app_label = "django_extensions"
        verbose_name = "lowercase alpha digits"


class RandomCharTestModelUppercaseAlphaDigits(models.Model):
    random_char_field = RandomCharField(length=8, uppercase=True)

    class Meta:
        app_label = "django_extensions"
        verbose_name = "uppercase alpha digits"


class RandomCharTestModelLowercase(models.Model):
    random_char_field = RandomCharField(length=8, lowercase=True, include_digits=False)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelUppercase(models.Model):
    random_char_field = RandomCharField(length=8, uppercase=True, include_digits=False)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelAlpha(models.Model):
    random_char_field = RandomCharField(length=8, include_digits=False)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelDigits(models.Model):
    random_char_field = RandomCharField(length=8, include_alpha=False)

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelPunctuation(models.Model):
    random_char_field = RandomCharField(
        length=8,
        include_punctuation=True,
        include_digits=False,
        include_alpha=False,
    )

    class Meta:
        app_label = "django_extensions"


class RandomCharTestModelUniqueTogether(models.Model):
    random_char_field = RandomCharField(length=8)
    common_field = models.CharField(max_length=10)

    class Meta:
        app_label = "django_extensions"
        unique_together = ("random_char_field", "common_field")


class TimestampedTestModel(TimeStampedModel):
    class Meta:
        app_label = "django_extensions"


class UnicodeVerboseNameModel(models.Model):
    cafe = models.IntegerField(verbose_name="café")
    parent_cafe = models.ForeignKey(
        "self",
        related_name="children",
        on_delete=models.CASCADE,
        verbose_name="café latte",
    )

    class Meta:
        app_label = "django_extensions"
        verbose_name = "café unicode model"


class Permission(models.Model):
    text = models.CharField(max_length=32)
    person = models.ForeignKey(Person, on_delete=models.CASCADE)


class UniqueTestAppModel(models.Model):
    global_id = models.CharField(max_length=32, unique=True)


class SqlDiff(models.Model):
    number = models.CharField(max_length=40, null=True, verbose_name="Chargennummer")
    creator = models.CharField(max_length=20, null=True, blank=True)

    class Meta:
        indexes = [models.Index(fields=["number", "creator"])]


class SqlDiffIndexes(models.Model):
    first = models.CharField(max_length=40, null=True, verbose_name="Chargennummer")
    second = models.CharField(max_length=20, null=True, blank=True)

    class Meta:
        indexes = [
            models.Index(fields=["first", "second"]),
        ]


class SqlDiffUniqueTogether(models.Model):
    aaa = models.CharField(max_length=20)
    bbb = models.CharField(max_length=20)

    class Meta:
        unique_together = ["aaa", "bbb"]


class Photo(models.Model):
    photo = models.FileField()


class CustomModelModificationDateTimeField(models.Model):
    field_to_update = models.BooleanField(default=True)
    custom_modified = ModificationDateTimeField()

    class Meta:
        app_label = "django_extensions"


class ModelModificationDateTimeField(models.Model):
    field_to_update = models.BooleanField(default=True)
    modified = ModificationDateTimeField()

    class Meta:
        app_label = "django_extensions"


class DisabledUpdateModelModificationDateTimeField(models.Model):
    field_to_update = models.BooleanField(default=True)
    modified = ModificationDateTimeField()

    update_modified = False

    class Meta:
        app_label = "django_extensions"


class HasOwnerModel(models.Model):
    content = models.TextField(default="")
    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)

    class Meta:
        app_label = "django_extensions"


class MultipleFieldsAndMethods(models.Model):
    char_field = models.CharField(max_length=10)
    integer_field = models.IntegerField()
    foreign_key_field = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)

    def has_self_only(self):
        pass

    def has_one_extra_argument(self, arg_one):
        pass

    def has_two_extra_arguments(self, arg_one, arg_two):
        pass

    def has_args_kwargs(self, *args, **kwargs):
        pass

    def has_defaults(self, one=1, two="Two", true=True, false=False, none=None):
        pass

    class Meta:
        app_label = "django_extensions"


def dummy_handler(sender, instance, **kwargs):
    pass


pre_save.connect(dummy_handler, sender=HasOwnerModel)



# -*- coding: utf-8 -*-
from io import StringIO

from django.core.management import CommandError, call_command
from django.test import TestCase
from django.test.utils import override_settings

from unittest import mock


class ResetSchemaExceptionsTests(TestCase):
    """Tests if reset_schema command raises exceptions."""

    def test_should_raise_CommandError_when_database_does_not_exist(self):
        with self.assertRaisesRegex(
            CommandError, "Unknown database non-existing_database"
        ):
            call_command("reset_schema", "--database=non-existing_database")

    @override_settings(
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.mysql",
            },
        }
    )
    def test_should_raise_CommandError_when_database_ENGINE_different_thant_postgresql(
        self,
    ):
        with self.assertRaisesRegex(
            CommandError, "This command can be used only with PostgreSQL databases."
        ):
            call_command("reset_schema")


@override_settings(
    DATABASES={
        "default": {
            "ENGINE": "django.db.backends.postgresql_psycopg2",
            "NAME": "test",
            "USER": "test",
            "PASSWORD": "test",
            "HOST": "localhost",
        },
    }
)
class ResetSchemaTests(TestCase):
    """Tests for reset_chema command."""

    def test_should_drop_schema_and_create_new_one(self):
        m_cursor = mock.Mock()
        m_router = mock.Mock()
        m_router.cursor.return_value = mock.Mock(
            __enter__=mock.Mock(return_value=m_cursor),
            __exit__=mock.Mock(return_value=False),
        )
        expected_calls = [
            mock.call("DROP SCHEMA test_public CASCADE"),
            mock.call("CREATE SCHEMA test_public"),
        ]

        with mock.patch(
            "django_extensions.management.commands.reset_schema.connections",
            {"default": m_router},
        ):
            call_command("reset_schema", "--noinput", "--schema=test_public")

        m_cursor.execute.assert_has_calls(expected_calls, any_order=False)

    @mock.patch("sys.stdout", new_callable=StringIO)
    @mock.patch("django_extensions.management.commands.reset_schema.input")
    def test_should_cancel_reset_schema_and_print_info_if_input_is_different_than_yes(
        self, m_input, m_stdout
    ):
        m_input.return_value = "no"

        call_command("reset_schema")

        self.assertEqual("Reset cancelled.\n", m_stdout.getvalue())



# -*- coding: utf-8 -*-
from io import StringIO

from django.urls import path
from django.core.management import CommandError, call_command
from django.http import HttpResponse
from django.test import TestCase
from django.test.utils import override_settings
from django.views.generic.base import View

from unittest.mock import Mock, patch


def function_based_view(request):
    pass


class ClassView(View):
    pass


urlpatterns = [
    path("lambda/view", lambda request: HttpResponse("OK")),
    path("function/based/", function_based_view, name="function-based-view"),
    path("class/based/", ClassView.as_view(), name="class-based-view"),
]


class ShowUrlsExceptionsTests(TestCase):
    """Tests if show_urls command raises exceptions."""

    def test_should_raise_CommandError_when_format_style_does_not_exists(self):
        with self.assertRaisesRegex(
            CommandError,
            "Format style 'invalid_format' does not exist. Options: aligned, dense, json, pretty-json, table, verbose",
        ):
            call_command("show_urls", "--format=invalid_format")

    def test_should_raise_CommandError_when_doesnt_have_urlconf_attr(self):
        with self.assertRaisesRegex(
            CommandError,
            'Settings module <Settings "tests.testapp.settings"> does not have the attribute INVALID_URLCONF.',
        ):
            call_command("show_urls", "--urlconf=INVALID_URLCONF")

    @override_settings(INVALID_URLCONF="")
    def test_should_raise_CommandError_when_doesnt_have_urlconf_attr_print_exc(self):
        m_traceback = Mock()
        with self.assertRaisesRegex(
            CommandError, "Error occurred while trying to load : Empty module name"
        ):
            with patch.dict("sys.modules", traceback=m_traceback):
                call_command("show_urls", "--urlconf=INVALID_URLCONF", "--traceback")

        self.assertTrue(m_traceback.print_exc.called)


@override_settings(ROOT_URLCONF="tests.management.commands.test_show_urls")
class ShowUrlsTests(TestCase):
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_unsorted_but_same_order_as_found_in_url_patterns(
        self, m_stdout
    ):
        call_command("show_urls", "-u", verbosity=3)

        lines = m_stdout.getvalue().splitlines()
        self.assertIn(
            "/lambda/view\ttests.management.commands.test_show_urls.<lambda>", lines[0]
        )
        self.assertIn(
            "/function/based/\ttests.management.commands.test_show_urls.function_based_view\tfunction-based-view",
            lines[1],
        )
        self.assertIn(
            "/class/based/\ttests.management.commands.test_show_urls.ClassView\tclass-based-view",
            lines[2],
        )

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_sorted_alphabetically(self, m_stdout):
        call_command("show_urls", verbosity=3)

        lines = m_stdout.getvalue().splitlines()
        self.assertEqual(
            "/class/based/\ttests.management.commands.test_show_urls.ClassView\tclass-based-view",
            lines[0],
        )
        self.assertEqual(
            "/function/based/\ttests.management.commands.test_show_urls.function_based_view\tfunction-based-view",
            lines[1],
        )
        self.assertEqual(
            "/lambda/view\ttests.management.commands.test_show_urls.<lambda>", lines[2]
        )

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_in_json_format(self, m_stdout):
        call_command("show_urls", "--format=json")

        self.assertJSONEqual(
            m_stdout.getvalue(),
            [
                {
                    "url": "/lambda/view",
                    "module": "tests.management.commands.test_show_urls.<lambda>",
                    "name": "",
                    "decorators": "",
                },
                {
                    "url": "/function/based/",
                    "module": "tests.management.commands.test_show_urls.function_based_view",
                    "name": "function-based-view",
                    "decorators": "",
                },
                {
                    "url": "/class/based/",
                    "module": "tests.management.commands.test_show_urls.ClassView",
                    "name": "class-based-view",
                    "decorators": "",
                },
            ],
        )
        self.assertEqual(len(m_stdout.getvalue().splitlines()), 1)

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_in_pretty_json_format(self, m_stdout):
        call_command("show_urls", "--format=pretty-json")

        self.assertJSONEqual(
            m_stdout.getvalue(),
            [
                {
                    "url": "/lambda/view",
                    "module": "tests.management.commands.test_show_urls.<lambda>",
                    "name": "",
                    "decorators": "",
                },
                {
                    "url": "/function/based/",
                    "module": "tests.management.commands.test_show_urls.function_based_view",
                    "name": "function-based-view",
                    "decorators": "",
                },
                {
                    "url": "/class/based/",
                    "module": "tests.management.commands.test_show_urls.ClassView",
                    "name": "class-based-view",
                    "decorators": "",
                },
            ],
        )
        self.assertEqual(len(m_stdout.getvalue().splitlines()), 20)

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_in_table_format(self, m_stdout):
        call_command("show_urls", "--format=table")

        self.assertIn(
            "/class/based/    | tests.management.commands.test_show_urls.ClassView           | class-based-view    |",
            m_stdout.getvalue(),
        )
        self.assertIn(
            "/function/based/ | tests.management.commands.test_show_urls.function_based_view | function-based-view |",
            m_stdout.getvalue(),
        )
        self.assertIn(
            "/lambda/view     | tests.management.commands.test_show_urls.<lambda>            |                     |",
            m_stdout.getvalue(),
        )

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_in_aligned_format(self, m_stdout):
        call_command("show_urls", "--format=aligned")

        lines = m_stdout.getvalue().splitlines()
        self.assertEqual(
            "/class/based/      tests.management.commands.test_show_urls.ClassView             class-based-view      ",
            lines[0],
        )
        self.assertEqual(
            "/function/based/   tests.management.commands.test_show_urls.function_based_view   function-based-view   ",
            lines[1],
        )
        self.assertEqual(
            "/lambda/view       tests.management.commands.test_show_urls.<lambda>                                    ",
            lines[2],
        )

    @patch("sys.stdout", new_callable=StringIO)
    def test_should_show_urls_with_no_color_option(self, m_stdout):
        call_command("show_urls", "--no-color")

        lines = m_stdout.getvalue().splitlines()
        self.assertEqual(
            "/class/based/\ttests.management.commands.test_show_urls.ClassView\tclass-based-view",
            lines[0],
        )
        self.assertEqual(
            "/function/based/\ttests.management.commands.test_show_urls.function_based_view\tfunction-based-view",
            lines[1],
        )
        self.assertEqual(
            "/lambda/view\ttests.management.commands.test_show_urls.<lambda>", lines[2]
        )



# -*- coding: utf-8 -*-
from django_extensions.management.jobs import DailyJob

from unittest import mock


DAILY_JOB_MOCK = mock.MagicMock()


class Job(DailyJob):
    help = "My sample daily job."

    def execute(self):
        DAILY_JOB_MOCK()



# -*- coding: utf-8 -*-
import logging
import os
import re
import socket
import sys
import traceback
import webbrowser
import functools
from pathlib import Path
from typing import List, Set  # NOQA

import django
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError, SystemCheckError
from django.core.management.color import color_style
from django.core.servers.basehttp import get_internal_wsgi_application
from django.dispatch import Signal
from django.template.autoreload import get_template_directories, reset_loaders
from django.utils.autoreload import file_changed, get_reloader
from django.views import debug as django_views_debug

try:
    if "whitenoise.runserver_nostatic" in settings.INSTALLED_APPS:
        USE_STATICFILES = False
    else:
        from django.contrib.staticfiles.handlers import StaticFilesHandler

        USE_STATICFILES = True
except ImportError:
    USE_STATICFILES = False

try:
    from werkzeug import run_simple
    from werkzeug.debug import DebuggedApplication
    from werkzeug.serving import WSGIRequestHandler as _WSGIRequestHandler
    from werkzeug.serving import make_ssl_devcert
    from werkzeug._internal import _log  # type: ignore
    from werkzeug import _reloader

    HAS_WERKZEUG = True
except ImportError:
    HAS_WERKZEUG = False

try:
    import OpenSSL  # NOQA

    HAS_OPENSSL = True
except ImportError:
    HAS_OPENSSL = False

from django_extensions.management.technical_response import null_technical_500_response
from django_extensions.management.utils import (
    RedirectHandler,
    has_ipdb,
    setup_logger,
    signalcommand,
)
from django_extensions.management.debug_cursor import monkey_patch_cursordebugwrapper


runserver_plus_started = Signal()
naiveip_re = re.compile(
    r"""^(?:
(?P<addr>
    (?P<ipv4>\d{1,3}(?:\.\d{1,3}){3}) |         # IPv4 address
    (?P<ipv6>\[[a-fA-F0-9:]+\]) |               # IPv6 address
    (?P<fqdn>[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*) # FQDN
):)?(?P<port>\d+)$""",
    re.X,
)
# 7-bit C1 ANSI sequences (https://stackoverflow.com/questions/14693701/how-can-i-remove-the-ansi-escape-sequences-from-a-string-in-python)
ansi_escape = re.compile(
    r"""
    \x1B  # ESC
    (?:   # 7-bit C1 Fe (except CSI)
        [@-Z\\-_]
    |     # or [ for CSI, followed by a control sequence
        \[
        [0-?]*  # Parameter bytes
        [ -/]*  # Intermediate bytes
        [@-~]   # Final byte
    )
""",
    re.VERBOSE,
)
DEFAULT_PORT = "8000"
DEFAULT_POLLER_RELOADER_INTERVAL = getattr(
    settings, "RUNSERVERPLUS_POLLER_RELOADER_INTERVAL", 1
)
DEFAULT_POLLER_RELOADER_TYPE = getattr(
    settings, "RUNSERVERPLUS_POLLER_RELOADER_TYPE", "auto"
)

logger = logging.getLogger(__name__)
_error_files = set()  # type: Set[str]


def get_all_template_files() -> Set[str]:
    template_list = set()

    for template_dir in get_template_directories():
        for base_dir, _, filenames in os.walk(template_dir):
            for filename in filenames:
                template_list.add(os.path.join(base_dir, filename))

    return template_list


if HAS_WERKZEUG:
    # Monkey patch the reloader to support adding more files to extra_files
    for name, reloader_loop_klass in _reloader.reloader_loops.items():

        class WrappedReloaderLoop(reloader_loop_klass):  # type: ignore
            def __init__(self, *args, **kwargs):
                self._template_files: Set[str] = get_all_template_files()
                super().__init__(*args, **kwargs)
                self._extra_files = self.extra_files

            @property
            def extra_files(self):
                template_files = get_all_template_files()

                # reset loaders if there are new files detected
                if len(self._template_files) != len(template_files):
                    changed = template_files.difference(self._template_files)
                    for filename in changed:
                        _log(
                            "info",
                            f" * New file {filename} added, reset template loaders",
                        )
                        self.register_file_changed(filename)

                    reset_loaders()

                self._template_files = template_files

                return self._extra_files.union(_error_files, template_files)

            @extra_files.setter
            def extra_files(self, extra_files):
                self._extra_files = extra_files

            def trigger_reload(self, filename: str) -> None:
                path = Path(filename)
                results = file_changed.send(sender=self, file_path=path)
                if not any(res[1] for res in results):
                    super().trigger_reload(filename)
                else:
                    _log(
                        "info",
                        f" * Detected change in {filename!r}, reset template loaders",
                    )
                    self.register_file_changed(filename)

            def register_file_changed(self, filename):
                if hasattr(self, "mtimes"):
                    mtime = os.stat(filename).st_mtime
                    self.mtimes[filename] = mtime

        _reloader.reloader_loops[name] = WrappedReloaderLoop


def gen_filenames():
    return get_reloader().watched_files()


def check_errors(fn):
    # Inspired by https://github.com/django/django/blob/master/django/utils/autoreload.py
    @functools.wraps(fn)
    def wrapper(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except Exception:
            _exception = sys.exc_info()

            _, ev, tb = _exception

            if getattr(ev, "filename", None) is None:
                # get the filename from the last item in the stack
                filename = traceback.extract_tb(tb)[-1][0]
            else:
                filename = ev.filename

            if filename not in _error_files:
                _error_files.add(filename)

            raise

    return wrapper


class Command(BaseCommand):
    help = "Starts a lightweight Web server for development."

    # Validation is called explicitly each time the server is reloaded.
    requires_system_checks: List[str] = []
    DEFAULT_CRT_EXTENSION = ".crt"
    DEFAULT_KEY_EXTENSION = ".key"

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "addrport", nargs="?", help="Optional port number, or ipaddr:port"
        )
        parser.add_argument(
            "--ipv6",
            "-6",
            action="store_true",
            dest="use_ipv6",
            default=False,
            help="Tells Django to use a IPv6 address.",
        )
        parser.add_argument(
            "--noreload",
            action="store_false",
            dest="use_reloader",
            default=True,
            help="Tells Django to NOT use the auto-reloader.",
        )
        parser.add_argument(
            "--browser",
            action="store_true",
            dest="open_browser",
            help="Tells Django to open a browser.",
        )
        parser.add_argument(
            "--nothreading",
            action="store_false",
            dest="threaded",
            help="Do not run in multithreaded mode.",
        )
        parser.add_argument(
            "--threaded",
            action="store_true",
            dest="threaded",
            help="Run in multithreaded mode.",
        )
        parser.add_argument(
            "--output",
            dest="output_file",
            default=None,
            help="Specifies an output file to send a copy of all messages "
            "(not flushed immediately).",
        )
        parser.add_argument(
            "--print-sql",
            action="store_true",
            default=False,
            help="Print SQL queries as they're executed",
        )
        parser.add_argument(
            "--truncate-sql",
            action="store",
            type=int,
            help="Truncate SQL queries to a number of characters.",
        )
        parser.add_argument(
            "--print-sql-location",
            action="store_true",
            default=False,
            help="Show location in code where SQL query generated from",
        )
        cert_group = parser.add_mutually_exclusive_group()
        cert_group.add_argument(
            "--cert",
            dest="cert_path",
            action="store",
            type=str,
            help="Deprecated alias for --cert-file option.",
        )
        cert_group.add_argument(
            "--cert-file",
            dest="cert_path",
            action="store",
            type=str,
            help="SSL .crt file path. If not provided path from --key-file will be "
            "selected. Either --cert-file or --key-file must be provided to use SSL.",
        )
        parser.add_argument(
            "--key-file",
            dest="key_file_path",
            action="store",
            type=str,
            help="SSL .key file path. If not provided path from --cert-file "
            "will be selected. Either --cert-file or --key-file must be provided "
            "to use SSL.",
        )
        parser.add_argument(
            "--extra-file",
            dest="extra_files",
            action="append",
            type=str,
            default=[],
            help="auto-reload whenever the given file changes too"
            " (can be specified multiple times)",
        )
        parser.add_argument(
            "--exclude-pattern",
            dest="exclude_patterns",
            action="append",
            type=str,
            default=[],
            help="ignore reload on changes to files matching this pattern"
            " (can be specified multiple times)",
        )
        parser.add_argument(
            "--reloader-interval",
            dest="reloader_interval",
            action="store",
            type=int,
            default=DEFAULT_POLLER_RELOADER_INTERVAL,
            help="After how many seconds auto-reload should scan for updates"
            " in poller-mode [default=%s]" % DEFAULT_POLLER_RELOADER_INTERVAL,
        )
        parser.add_argument(
            "--reloader-type",
            dest="reloader_type",
            action="store",
            type=str,
            default=DEFAULT_POLLER_RELOADER_TYPE,
            help="Werkzeug reloader type "
            "[options are auto, watchdog, or stat, default=%s]"
            % DEFAULT_POLLER_RELOADER_TYPE,
        )
        parser.add_argument(
            "--pdb",
            action="store_true",
            dest="pdb",
            default=False,
            help="Drop into pdb shell at the start of any view.",
        )
        parser.add_argument(
            "--ipdb",
            action="store_true",
            dest="ipdb",
            default=False,
            help="Drop into ipdb shell at the start of any view.",
        )
        parser.add_argument(
            "--pm",
            action="store_true",
            dest="pm",
            default=False,
            help="Drop into (i)pdb shell if an exception is raised in a view.",
        )
        parser.add_argument(
            "--startup-messages",
            dest="startup_messages",
            action="store",
            default="reload",
            help="When to show startup messages: "
            "reload [default], once, always, never.",
        )
        parser.add_argument(
            "--keep-meta-shutdown",
            dest="keep_meta_shutdown_func",
            action="store_true",
            default=False,
            help="Keep request.META['werkzeug.server.shutdown'] function which is "
            "automatically removed because Django debug pages tries to call the "
            "function and unintentionally shuts down the Werkzeug server.",
        )
        parser.add_argument(
            "--nopin",
            dest="nopin",
            action="store_true",
            default=False,
            help="Disable the PIN in werkzeug. USE IT WISELY!",
        )

        if USE_STATICFILES:
            parser.add_argument(
                "--nostatic",
                action="store_false",
                dest="use_static_handler",
                default=True,
                help="Tells Django to NOT automatically serve static files.",
            )
            parser.add_argument(
                "--insecure",
                action="store_true",
                dest="insecure_serving",
                default=False,
                help="Allows serving static files even if DEBUG is False.",
            )

    @signalcommand
    def handle(self, *args, **options):
        addrport = options["addrport"]
        startup_messages = options["startup_messages"]
        if startup_messages == "reload":
            self.show_startup_messages = os.environ.get("RUNSERVER_PLUS_SHOW_MESSAGES")
        elif startup_messages == "once":
            self.show_startup_messages = not os.environ.get(
                "RUNSERVER_PLUS_SHOW_MESSAGES"
            )
        elif startup_messages == "never":
            self.show_startup_messages = False
        else:
            self.show_startup_messages = True

        os.environ["RUNSERVER_PLUS_SHOW_MESSAGES"] = "1"

        setup_logger(
            logger, self.stderr, filename=options["output_file"]
        )  # , fmt="[%(name)s] %(message)s")
        logredirect = RedirectHandler(__name__)

        # Redirect werkzeug log items
        werklogger = logging.getLogger("werkzeug")
        werklogger.setLevel(logging.INFO)
        werklogger.addHandler(logredirect)
        werklogger.propagate = False

        pdb_option = options["pdb"]
        ipdb_option = options["ipdb"]
        pm = options["pm"]
        try:
            from django_pdb.middleware import PdbMiddleware
        except ImportError:
            if pdb_option or ipdb_option or pm:
                raise CommandError(
                    "django-pdb is required for --pdb, --ipdb and --pm options. "
                    "Please visit https://pypi.python.org/pypi/django-pdb or install "
                    "via pip. (pip install django-pdb)"
                )
            pm = False
        else:
            # Add pdb middleware if --pdb is specified or if in DEBUG mode
            if pdb_option or ipdb_option or settings.DEBUG:
                middleware = "django_pdb.middleware.PdbMiddleware"
                settings_middleware = (
                    getattr(settings, "MIDDLEWARE", None) or settings.MIDDLEWARE_CLASSES
                )

                if middleware not in settings_middleware:
                    if isinstance(settings_middleware, tuple):
                        settings_middleware += (middleware,)
                    else:
                        settings_middleware += [middleware]

            # If --pdb is specified then always break at the start of views.
            # Otherwise break only if a 'pdb' query parameter is set in the url
            if pdb_option:
                PdbMiddleware.always_break = "pdb"
            elif ipdb_option:
                PdbMiddleware.always_break = "ipdb"

            def postmortem(request, exc_type, exc_value, tb):
                if has_ipdb():
                    import ipdb

                    p = ipdb
                else:
                    import pdb

                    p = pdb
                print(
                    "Exception occured: %s, %s" % (exc_type, exc_value), file=sys.stderr
                )
                p.post_mortem(tb)

        # usurp django's handler
        django_views_debug.technical_500_response = (
            postmortem if pm else null_technical_500_response
        )

        self.use_ipv6 = options["use_ipv6"]
        if self.use_ipv6 and not socket.has_ipv6:
            raise CommandError("Your Python does not support IPv6.")
        self._raw_ipv6 = False
        if not addrport:
            try:
                addrport = settings.RUNSERVERPLUS_SERVER_ADDRESS_PORT
            except AttributeError:
                pass
        if not addrport:
            self.addr = ""
            self.port = DEFAULT_PORT
        else:
            m = re.match(naiveip_re, addrport)
            if m is None:
                raise CommandError(
                    '"%s" is not a valid port number or address:port pair.' % addrport
                )
            self.addr, _ipv4, _ipv6, _fqdn, self.port = m.groups()
            if not self.port.isdigit():
                raise CommandError("%r is not a valid port number." % self.port)
            if self.addr:
                if _ipv6:
                    self.addr = self.addr[1:-1]
                    self.use_ipv6 = True
                    self._raw_ipv6 = True
                elif self.use_ipv6 and not _fqdn:
                    raise CommandError('"%s" is not a valid IPv6 address.' % self.addr)
        if not self.addr:
            self.addr = "::1" if self.use_ipv6 else "127.0.0.1"
            self._raw_ipv6 = True

        truncate = None if options["truncate_sql"] == 0 else options["truncate_sql"]

        with monkey_patch_cursordebugwrapper(
            print_sql=options["print_sql"],
            print_sql_location=options["print_sql_location"],
            truncate=truncate,
            logger=logger.info,
            confprefix="RUNSERVER_PLUS",
        ):
            self.inner_run(options)

    def get_handler(self, *args, **options):
        """Return the default WSGI handler for the runner."""
        return get_internal_wsgi_application()

    def get_error_handler(self, exc, **options):
        def application(env, start_response):
            if isinstance(exc, SystemCheckError):
                error_message = ansi_escape.sub("", str(exc))
                raise SystemCheckError(error_message)

            raise exc

        return application

    def inner_run(self, options):
        if not HAS_WERKZEUG:
            raise CommandError(
                "Werkzeug is required to use runserver_plus. "
                "Please visit https://werkzeug.palletsprojects.com/ or install via pip."
                " (pip install Werkzeug)"
            )

        # Set colored output
        if settings.DEBUG:
            try:
                set_werkzeug_log_color()
            except (
                Exception
            ):  # We are dealing with some internals, anything could go wrong
                if self.show_startup_messages:
                    print(
                        "Wrapping internal werkzeug logger "
                        "for color highlighting has failed!"
                    )

        class WSGIRequestHandler(_WSGIRequestHandler):
            def make_environ(self):
                environ = super().make_environ()
                if (
                    not options["keep_meta_shutdown_func"]
                    and "werkzeug.server.shutdown" in environ
                ):
                    del environ["werkzeug.server.shutdown"]
                remote_user = os.getenv("REMOTE_USER")
                if remote_user is not None:
                    environ["REMOTE_USER"] = remote_user
                return environ

        threaded = options["threaded"]
        use_reloader = options["use_reloader"]
        open_browser = options["open_browser"]
        quit_command = "CONTROL-C" if sys.platform != "win32" else "CTRL-BREAK"
        reloader_interval = options["reloader_interval"]
        reloader_type = options["reloader_type"]
        self.extra_files = set(options["extra_files"])
        exclude_patterns = set(options["exclude_patterns"])

        self.nopin = options["nopin"]

        if self.show_startup_messages:
            print("Performing system checks...\n")

        try:
            check_errors(self.check)(display_num_errors=self.show_startup_messages)
            check_errors(self.check_migrations)()
            handler = check_errors(self.get_handler)(**options)
        except Exception as exc:
            self.stderr.write("Error occurred during checks: %r" % exc, ending="\n\n")
            handler = self.get_error_handler(exc, **options)

        if USE_STATICFILES:
            use_static_handler = options["use_static_handler"]
            insecure_serving = options["insecure_serving"]
            if use_static_handler and (settings.DEBUG or insecure_serving):
                handler = StaticFilesHandler(handler)

        if options["cert_path"] or options["key_file_path"]:
            if not HAS_OPENSSL:
                raise CommandError(
                    "Python OpenSSL Library is "
                    "required to use runserver_plus with ssl support. "
                    "Install via pip (pip install pyOpenSSL)."
                )

            certfile, keyfile = self.determine_ssl_files_paths(options)
            dir_path, root = os.path.split(certfile)
            root, _ = os.path.splitext(root)
            try:
                if os.path.exists(certfile) and os.path.exists(keyfile):
                    ssl_context = (certfile, keyfile)
                else:  # Create cert, key files ourselves.
                    ssl_context = make_ssl_devcert(
                        os.path.join(dir_path, root), host="localhost"
                    )
            except ImportError:
                if self.show_startup_messages:
                    print(
                        "Werkzeug version is less than 0.9, trying adhoc certificate."
                    )
                ssl_context = "adhoc"
        else:
            ssl_context = None

        bind_url = "%s://%s:%s/" % (
            "https" if ssl_context else "http",
            self.addr if not self._raw_ipv6 else "[%s]" % self.addr,
            self.port,
        )

        if self.show_startup_messages:
            print(
                "\nDjango version %s, using settings %r"
                % (django.get_version(), settings.SETTINGS_MODULE)
            )
            print("Development server is running at %s" % (bind_url,))
            print("Using the Werkzeug debugger (https://werkzeug.palletsprojects.com/)")
            print("Quit the server with %s." % quit_command)

        if open_browser:
            webbrowser.open(bind_url)

        if use_reloader and settings.USE_I18N:
            self.extra_files |= set(
                filter(lambda filename: str(filename).endswith(".mo"), gen_filenames())
            )

        if getattr(settings, "RUNSERVER_PLUS_EXTRA_FILES", []):
            self.extra_files |= set(settings.RUNSERVER_PLUS_EXTRA_FILES)

        exclude_patterns |= set(
            getattr(settings, "RUNSERVER_PLUS_EXCLUDE_PATTERNS", [])
        )

        # Werkzeug needs to be clued in its the main instance if running
        # without reloader or else it won't show key.
        # https://git.io/vVIgo
        if not use_reloader:
            os.environ["WERKZEUG_RUN_MAIN"] = "true"

        # Don't run a second instance of the debugger / reloader
        # See also: https://github.com/django-extensions/django-extensions/issues/832
        if os.environ.get("WERKZEUG_RUN_MAIN") != "true":
            if self.nopin:
                os.environ["WERKZEUG_DEBUG_PIN"] = "off"
            handler = DebuggedApplication(handler, True)

        runserver_plus_started.send(sender=self)
        run_simple(
            self.addr,
            int(self.port),
            handler,
            use_reloader=use_reloader,
            use_debugger=True,
            extra_files=self.extra_files,
            exclude_patterns=exclude_patterns,
            reloader_interval=reloader_interval,
            reloader_type=reloader_type,
            threaded=threaded,
            request_handler=WSGIRequestHandler,
            ssl_context=ssl_context,
        )

    @classmethod
    def determine_ssl_files_paths(cls, options):
        key_file_path = os.path.expanduser(options.get("key_file_path") or "")
        cert_path = os.path.expanduser(options.get("cert_path") or "")
        cert_file = cls._determine_path_for_file(
            cert_path, key_file_path, cls.DEFAULT_CRT_EXTENSION
        )
        key_file = cls._determine_path_for_file(
            key_file_path, cert_path, cls.DEFAULT_KEY_EXTENSION
        )
        return cert_file, key_file

    @classmethod
    def _determine_path_for_file(
        cls, current_file_path, other_file_path, expected_extension
    ):
        directory = cls._get_directory_basing_on_file_paths(
            current_file_path, other_file_path
        )
        file_name = cls._get_file_name(current_file_path) or cls._get_file_name(
            other_file_path
        )
        extension = cls._get_extension(current_file_path) or expected_extension
        return os.path.join(directory, file_name + extension)

    @classmethod
    def _get_directory_basing_on_file_paths(cls, current_file_path, other_file_path):
        return (
            cls._get_directory(current_file_path)
            or cls._get_directory(other_file_path)
            or os.getcwd()
        )

    @classmethod
    def _get_directory(cls, file_path):
        return os.path.split(file_path)[0]

    @classmethod
    def _get_file_name(cls, file_path):
        return os.path.splitext(os.path.split(file_path)[1])[0]

    @classmethod
    def _get_extension(cls, file_path):
        return os.path.splitext(file_path)[1]


def set_werkzeug_log_color():
    """Try to set color to the werkzeug log."""
    _style = color_style()
    _orig_log = _WSGIRequestHandler.log

    def werk_log(self, type, message, *args):
        try:
            msg = "%s - - [%s] %s" % (
                self.address_string(),
                self.log_date_time_string(),
                message % args,
            )
            http_code = str(args[1])
        except Exception:
            return _orig_log(type, message, *args)

        # Utilize terminal colors, if available
        if http_code[0] == "2":
            # Put 2XX first, since it should be the common case
            msg = _style.HTTP_SUCCESS(msg)
        elif http_code[0] == "1":
            msg = _style.HTTP_INFO(msg)
        elif http_code == "304":
            msg = _style.HTTP_NOT_MODIFIED(msg)
        elif http_code[0] == "3":
            msg = _style.HTTP_REDIRECT(msg)
        elif http_code == "404":
            msg = _style.HTTP_NOT_FOUND(msg)
        elif http_code[0] == "4":
            msg = _style.HTTP_BAD_REQUEST(msg)
        else:
            # Any 5XX, or any other response
            msg = _style.HTTP_SERVER_ERROR(msg)

        _log(type, msg)

    _WSGIRequestHandler.log = werk_log



# -*- coding: utf-8 -*-
from django_extensions.management.jobs import MonthlyJob

from unittest import mock


MONTHLY_JOB_MOCK = mock.MagicMock()


class Job(MonthlyJob):
    help = "My sample monthly job."

    def execute(self):
        MONTHLY_JOB_MOCK()



# -*- coding: utf-8 -*-
import pytest

from django.core.management import call_command

from django_extensions.management.commands.raise_test_exception import (
    DjangoExtensionsTestException,
)


def test_raise_test_exception():
    with pytest.raises(DjangoExtensionsTestException):
        call_command("raise_test_exception")



# -*- coding: utf-8 -*-
from io import StringIO

from django.core.management import CommandError, call_command
from django.test import TestCase
from django.test.utils import override_settings

from unittest.mock import patch


MYSQL_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.mysql",
    "NAME": "dbatabase",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "127.0.0.1",
    "PORT": "3306",
}

SQLITE3_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.sqlite3",
    "NAME": "db.sqlite3",
}

POSTGRESQL_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.postgresql",
    "NAME": "database",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "localhost",
    "PORT": "5432",
}

POSTGRESQL_PSYCOPG2_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.postgresql_psycopg2",
    "NAME": "database",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "localhost",
    "PORT": "5432",
}

POSTGIS_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.postgis",
    "NAME": "database",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "localhost",
}

POSTGIS_WITH_PORT_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.postgis",
    "NAME": "database",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "localhost",
    "PORT": 5432,
}


class SqlDsnExceptionsTests(TestCase):
    """Tests for sqldsn management command exceptions."""

    @override_settings(DATABASES={})
    def test_should_raise_CommandError_if_unknown_database_does_not_exist(self):
        with self.assertRaisesRegex(CommandError, "Unknown database unknown"):
            call_command("sqldsn", "--database=unknown")


class SqlDsnTests(TestCase):
    """Tests for sqldsn management command."""

    maxDiff = None

    @override_settings(DATABASES={"default": SQLITE3_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info_for_default_sqlite3_database(self, m_stdout):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.sqlite3':
db.sqlite3
"""
        call_command("sqldsn")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": SQLITE3_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_uri_for_default_sqlite3_database(self, m_stdout):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.sqlite3':
sqlite:///db.sqlite3
"""
        call_command("sqldsn", "--style=uri")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": MYSQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_quiet_info_for_mysql_database(self, m_stdout):
        expected_result = """host="127.0.0.1", db="dbatabase", user="foo", passwd="bar", port="3306"
"""
        call_command("sqldsn", "-q")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": MYSQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_quiet_uri_for_mysql_database(self, m_stdout):
        expected_result = """mysql://foo:bar@127.0.0.1:3306/dbatabase
"""
        call_command("sqldsn", "-q", "--style=uri")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": MYSQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_quiet_args_for_mysql_database(self, m_stdout):
        expected_result = """-h "127.0.0.1" -D "dbatabase" -u "foo" -p "bar" -P 3306
"""
        call_command("sqldsn", "-q", "--style=args")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_all_info_for_postgresql_database(self, m_stdout):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.postgresql':
host='localhost' dbname='database' user='foo' password='bar' port='5432'
"""

        call_command("sqldsn", "-a")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_PSYCOPG2_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info__with_kwargs_style_for_postgresql_psycopg2_database(
        self, m_stdout
    ):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.postgresql_psycopg2':
host='localhost', database='database', user='foo', password='bar', port='5432'
"""

        call_command("sqldsn", "--style=kwargs")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGIS_WITH_PORT_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info__with_uri_style_for_postgis_database(self, m_stdout):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.postgis':
postgresql://foo:bar@localhost:5432/database
"""

        call_command("sqldsn", "--style=uri")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGIS_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info__with_uri_style_without_port_for_postgis_database(
        self, m_stdout
    ):
        expected_result = """DSN for database 'default' with engine 'django.db.backends.postgis':
postgresql://foo:bar@localhost/database
"""

        call_command("sqldsn", "--style=uri")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info_with_pgpass_style_and_quiet_option_for_postgresql_database(
        self, m_stdout
    ):
        expected_result = "localhost:5432:database:foo:bar\n"

        call_command("sqldsn", "--style=pgpass", "-q")

        self.assertEqual(expected_result, m_stdout.getvalue())

    @override_settings(
        DATABASES={
            "default": POSTGRESQL_DATABASE_SETTINGS,
            "slave": MYSQL_DATABASE_SETTINGS,
            "test": SQLITE3_DATABASE_SETTINGS,
            "unknown": {
                "ENGINE": "django.db.backends.unknown",
            },
        }
    )
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info_for_all_databases(self, m_stdout):
        default_postgresql = """DSN for database 'default' with engine 'django.db.backends.postgresql':
host='localhost' dbname='database' user='foo' password='bar' port='5432'"""
        slave_mysql = '''DSN for database 'slave' with engine 'django.db.backends.mysql':
host="127.0.0.1", db="dbatabase", user="foo", passwd="bar", port="3306"'''
        test_sqlite3 = """DSN for database 'test' with engine 'django.db.backends.sqlite3':
db.sqlite3"""
        unknown = """DSN for database 'unknown' with engine 'django.db.backends.unknown':
Unknown database, can't generate DSN"""

        call_command("sqldsn", "--all")

        self.assertIn(default_postgresql, m_stdout.getvalue())
        self.assertIn(slave_mysql, m_stdout.getvalue())
        self.assertIn(test_sqlite3, m_stdout.getvalue())
        self.assertIn(unknown, m_stdout.getvalue())

    @override_settings(
        DATABASES={
            "default": POSTGRESQL_DATABASE_SETTINGS,
            "slave": MYSQL_DATABASE_SETTINGS,
            "test": SQLITE3_DATABASE_SETTINGS,
            "unknown": {
                "ENGINE": "django.db.backends.unknown",
            },
        }
    )
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_info_with_all_style_for_all_databases(self, m_stdout):
        default_postgresql = """DSN for database 'default' with engine 'django.db.backends.postgresql':
host='localhost' dbname='database' user='foo' password='bar' port='5432'
host='localhost', database='database', user='foo', password='bar', port='5432'
postgresql://foo:bar@localhost:5432/database
localhost:5432:database:foo:bar"""
        slave_mysql = '''DSN for database 'slave' with engine 'django.db.backends.mysql':
host="127.0.0.1", db="dbatabase", user="foo", passwd="bar", port="3306"'''
        test_sqlite3 = """DSN for database 'test' with engine 'django.db.backends.sqlite3':
db.sqlite3"""
        unknown = """DSN for database 'unknown' with engine 'django.db.backends.unknown':
Unknown database, can't generate DSN"""

        call_command("sqldsn", "--all", "--style=all")

        self.assertIn(default_postgresql, m_stdout.getvalue())
        self.assertIn(slave_mysql, m_stdout.getvalue())
        self.assertIn(test_sqlite3, m_stdout.getvalue())
        self.assertIn(unknown, m_stdout.getvalue())



# -*- coding: utf-8 -*-
"""
modelviz.py - DOT file generator for Django Models

Based on:
  Django model to DOT (Graphviz) converter
  by Antonio Cavedoni <antonio@cavedoni.org>
  Adapted to be used with django-extensions
"""

import datetime
import os
import re

from django.apps import apps
from django.db.models import deletion
from django.db.models.fields.related import (
    ForeignKey,
    ManyToManyField,
    OneToOneField,
    RelatedField,
)
from django.db.models.fields.reverse_related import (
    OneToOneRel,
    ManyToOneRel,
)
from django.contrib.contenttypes.fields import GenericRelation
from django.template import Context, Template, loader
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe
from django.utils.translation import activate as activate_language


__version__ = "1.1"
__license__ = "Python"
__author__ = ("Bas van Oostveen <v.oostveen@gmail.com>",)
__contributors__ = [
    "Antonio Cavedoni <https://cavedoni.com/>Stefano J. Attardi <https://attardi.org/>",
    "limodou",
    "Carlo C8E Miron",
    "Andre Campos <cahenan@gmail.com>",
    "Justin Findlay <jfindlay@gmail.com>",
    "Alexander Houben <alexander@houben.ch>",
    "Joern Hees <gitdev@joernhees.de>",
    "Kevin Cherepski <cherepski@gmail.com>",
    "Jose Tomas Tocino <theom3ga@gmail.com>",
    "Adam Dobrawy <naczelnik@jawnosc.tk>",
    "Mikkel Munch Mortensen <https://www.detfalskested.dk/>",
    "Andrzej Bistram <andrzej.bistram@gmail.com>",
    "Daniel Lipsitt <danlipsitt@gmail.com>",
    "Florian Anceau <flow.gunso@gmail.com>",
]


ON_DELETE_COLORS = {
    deletion.CASCADE: "red",
    deletion.PROTECT: "blue",
    deletion.SET_NULL: "orange",
    deletion.SET_DEFAULT: "green",
    deletion.SET: "yellow",
    deletion.DO_NOTHING: "grey",
    deletion.RESTRICT: "purple",
}


def parse_file_or_list(arg):
    if not arg:
        return []
    if isinstance(arg, (list, tuple, set)):
        return arg
    if "," not in arg and os.path.isfile(arg):
        return [e.strip() for e in open(arg).readlines()]
    return [e.strip() for e in arg.split(",")]


class ModelGraph:
    def __init__(self, app_labels, **kwargs):
        self.graphs = []
        self.cli_options = kwargs.get("cli_options", None)
        self.disable_fields = kwargs.get("disable_fields", False)
        self.disable_abstract_fields = kwargs.get("disable_abstract_fields", False)
        self.include_models = parse_file_or_list(kwargs.get("include_models", ""))
        self.all_applications = kwargs.get("all_applications", False)
        self.use_subgraph = kwargs.get("group_models", False)
        self.verbose_names = kwargs.get("verbose_names", False)
        self.inheritance = kwargs.get("inheritance", True)
        self.relations_as_fields = kwargs.get("relations_as_fields", True)
        self.relation_fields_only = kwargs.get("relation_fields_only", False)
        self.sort_fields = kwargs.get("sort_fields", True)
        self.language = kwargs.get("language", None)
        if self.language is not None:
            activate_language(self.language)
        self.exclude_columns = parse_file_or_list(kwargs.get("exclude_columns", ""))
        self.exclude_models = parse_file_or_list(kwargs.get("exclude_models", ""))
        self.hide_edge_labels = kwargs.get("hide_edge_labels", False)
        self.arrow_shape = kwargs.get("arrow_shape")
        self.color_code_deletions = kwargs.get("color_code_deletions", False)
        if self.all_applications:
            self.app_labels = [app.label for app in apps.get_app_configs()]
        else:
            self.app_labels = app_labels
        self.rankdir = kwargs.get("rankdir")
        self.display_field_choices = kwargs.get("display_field_choices", False)
        self.ordering = kwargs.get("ordering")

    def generate_graph_data(self):
        self.process_apps()

        nodes = []
        for graph in self.graphs:
            nodes.extend([e["name"] for e in graph["models"]])

        for graph in self.graphs:
            for model in graph["models"]:
                for relation in model["relations"]:
                    if relation is not None:
                        if relation["target"] in nodes:
                            relation["needs_node"] = False

    def get_graph_data(self, as_json=False):
        now = datetime.datetime.now()
        graph_data = {
            "created_at": now.strftime("%Y-%m-%d %H:%M"),
            "cli_options": self.cli_options,
            "disable_fields": self.disable_fields,
            "disable_abstract_fields": self.disable_abstract_fields,
            "display_field_choices": self.display_field_choices,
            "use_subgraph": self.use_subgraph,
            "rankdir": self.rankdir,
            "ordering": self.ordering,
        }

        if as_json:
            # We need to remove the model and field class
            # because it is not JSON serializable
            graphs = [context.flatten() for context in self.graphs]
            for context in graphs:
                for model_data in context["models"]:
                    model_data.pop("model")
                    for field_data in model_data["fields"]:
                        field_data.pop("field")
            graph_data["graphs"] = graphs
        else:
            graph_data["graphs"] = self.graphs

        return graph_data

    def add_attributes(self, field, abstract_fields):
        if self.verbose_names and field.verbose_name:
            label = force_str(field.verbose_name)
            if label.islower():
                label = label.capitalize()
        else:
            label = field.name

        t = type(field).__name__
        if isinstance(field, (OneToOneField, ForeignKey)):
            t += " ({0})".format(field.remote_field.field_name)
        if self.display_field_choices and field.choices is not None:
            choices = {c for c, _ in field.choices}
            t = str(choices)
        # TODO: ManyToManyField, GenericRelation

        return {
            "field": field,
            "name": field.name,
            "label": label,
            "type": t,
            "blank": field.blank,
            "abstract": any(
                field.creation_counter == abstract_field.creation_counter
                for abstract_field in abstract_fields
            ),
            "relation": isinstance(field, RelatedField),
            "primary_key": field.primary_key,
        }

    def add_relation(self, field, model, extras="", color=None):
        if self.verbose_names and field.verbose_name:
            label = force_str(field.verbose_name)
            if label.islower():
                label = label.capitalize()
        else:
            label = field.name

        # show related field name
        if hasattr(field, "related_query_name"):
            related_query_name = field.related_query_name()
            if self.verbose_names and related_query_name.islower():
                related_query_name = related_query_name.replace("_", " ").capitalize()
            label = "{} ({})".format(label, force_str(related_query_name))
        if self.hide_edge_labels:
            label = ""

        # handle self-relationships and lazy-relationships
        if isinstance(field.remote_field.model, str):
            if field.remote_field.model == "self":
                target_model = field.model
            else:
                if "." in field.remote_field.model:
                    app_label, model_name = field.remote_field.model.split(".", 1)
                else:
                    app_label = field.model._meta.app_label
                    model_name = field.remote_field.model
                target_model = apps.get_model(app_label, model_name)
        else:
            target_model = field.remote_field.model

        if color:
            extras = "[{}, color={}]".format(extras[1:-1], color)

        _rel = self.get_relation_context(target_model, field, label, extras)

        if _rel not in model["relations"] and self.use_model(_rel["target"]):
            return _rel

    def get_abstract_models(self, appmodels):
        abstract_models = []
        for appmodel in appmodels:
            abstract_models += [
                abstract_model
                for abstract_model in appmodel.__bases__
                if hasattr(abstract_model, "_meta") and abstract_model._meta.abstract
            ]
        abstract_models = list(set(abstract_models))  # remove duplicates
        return abstract_models

    def get_app_context(self, app):
        return Context(
            {
                "name": '"%s"' % app.name,
                "app_name": "%s" % app.name,
                "cluster_app_name": "cluster_%s" % app.name.replace(".", "_"),
                "models": [],
            }
        )

    def get_appmodel_attributes(self, appmodel):
        if self.relations_as_fields:
            attributes = [field for field in appmodel._meta.local_fields]
        else:
            # Find all the 'real' attributes. Relations are depicted as graph edges
            # instead of attributes
            attributes = [
                field
                for field in appmodel._meta.local_fields
                if not isinstance(field, RelatedField)
            ]
        return attributes

    def get_appmodel_abstracts(self, appmodel):
        return [
            abstract_model.__name__
            for abstract_model in appmodel.__bases__
            if hasattr(abstract_model, "_meta") and abstract_model._meta.abstract
        ]

    def get_appmodel_context(self, appmodel, appmodel_abstracts):
        context = {
            "model": appmodel,
            "app_name": appmodel.__module__.replace(".", "_"),
            "name": appmodel.__name__,
            "abstracts": appmodel_abstracts,
            "fields": [],
            "relations": [],
        }

        if self.verbose_names and appmodel._meta.verbose_name:
            context["label"] = force_str(appmodel._meta.verbose_name)
        else:
            context["label"] = context["name"]

        return context

    def get_bases_abstract_fields(self, c):
        _abstract_fields = []
        for e in c.__bases__:
            if hasattr(e, "_meta") and e._meta.abstract:
                _abstract_fields.extend(e._meta.fields)
                _abstract_fields.extend(self.get_bases_abstract_fields(e))
        return _abstract_fields

    def get_inheritance_context(self, appmodel, parent):
        label = "multi-table"
        if parent._meta.abstract:
            label = "abstract"
        if appmodel._meta.proxy:
            label = "proxy"
        label += r"\ninheritance"
        if self.hide_edge_labels:
            label = ""
        return {
            "target_app": parent.__module__.replace(".", "_"),
            "target": parent.__name__,
            "type": "inheritance",
            "name": "inheritance",
            "label": label,
            "arrows": "[arrowhead=empty, arrowtail=none, dir=both]",
            "needs_node": True,
        }

    def get_models(self, app):
        appmodels = list(app.get_models())
        return appmodels

    def get_relation_context(self, target_model, field, label, extras):
        return {
            "target_app": target_model.__module__.replace(".", "_"),
            "target": target_model.__name__,
            "type": type(field).__name__,
            "name": field.name,
            "label": label,
            "arrows": extras,
            "needs_node": True,
        }

    def process_attributes(self, field, model, pk, abstract_fields):
        newmodel = model.copy()
        if self.skip_field(field) or pk and field == pk:
            return newmodel
        newmodel["fields"].append(self.add_attributes(field, abstract_fields))
        return newmodel

    def process_apps(self):
        for app_label in self.app_labels:
            app = apps.get_app_config(app_label)
            if not app:
                continue
            app_graph = self.get_app_context(app)
            app_models = self.get_models(app)
            abstract_models = self.get_abstract_models(app_models)
            app_models = abstract_models + app_models

            for appmodel in app_models:
                if not self.use_model(appmodel._meta.object_name):
                    continue
                appmodel_abstracts = self.get_appmodel_abstracts(appmodel)
                abstract_fields = self.get_bases_abstract_fields(appmodel)
                model = self.get_appmodel_context(appmodel, appmodel_abstracts)
                attributes = self.get_appmodel_attributes(appmodel)

                # find primary key and print it first
                # ignoring implicit id if other pk exists
                pk = appmodel._meta.pk
                if pk and not appmodel._meta.abstract and pk in attributes:
                    model["fields"].append(self.add_attributes(pk, abstract_fields))

                for field in attributes:
                    model = self.process_attributes(field, model, pk, abstract_fields)

                if self.sort_fields:
                    model = self.sort_model_fields(model)

                for field in appmodel._meta.local_fields:
                    model = self.process_local_fields(field, model, abstract_fields)

                for field in appmodel._meta.local_many_to_many:
                    model = self.process_local_many_to_many(field, model)

                if self.inheritance:
                    # add inheritance arrows
                    for parent in appmodel.__bases__:
                        model = self.process_parent(parent, appmodel, model)

                app_graph["models"].append(model)
            if app_graph["models"]:
                self.graphs.append(app_graph)

    def process_local_fields(self, field, model, abstract_fields):
        newmodel = model.copy()
        if (
            field.attname.endswith("_ptr_id")
            or field in abstract_fields
            or self.skip_field(field)
        ):
            # excluding field redundant with inheritance relation
            # excluding fields inherited from abstract classes.
            # they too show as local_fields
            return newmodel

        color = None
        if self.color_code_deletions and isinstance(field, (OneToOneField, ForeignKey)):
            field_on_delete = getattr(field.remote_field, "on_delete", None)
            color = ON_DELETE_COLORS.get(field_on_delete)

        if isinstance(field, OneToOneField):
            relation = self.add_relation(
                field, newmodel, "[arrowhead=none, arrowtail=none, dir=both]", color
            )
        elif isinstance(field, ForeignKey):
            relation = self.add_relation(
                field,
                newmodel,
                "[arrowhead=none, arrowtail={}, dir=both]".format(self.arrow_shape),
                color,
            )
        else:
            relation = None
        if relation is not None:
            newmodel["relations"].append(relation)
        return newmodel

    def process_local_many_to_many(self, field, model):
        newmodel = model.copy()
        if self.skip_field(field):
            return newmodel
        relation = None
        if isinstance(field, ManyToManyField):
            if (
                hasattr(field.remote_field.through, "_meta")
                and field.remote_field.through._meta.auto_created
            ):
                relation = self.add_relation(
                    field,
                    newmodel,
                    "[arrowhead={} arrowtail={}, dir=both]".format(
                        self.arrow_shape, self.arrow_shape
                    ),
                )
        elif isinstance(field, GenericRelation):
            relation = self.add_relation(
                field,
                newmodel,
                mark_safe(
                    '[style="dotted", arrowhead=normal, arrowtail=normal, dir=both]'
                ),
            )
        if relation is not None:
            newmodel["relations"].append(relation)
        return newmodel

    def process_parent(self, parent, appmodel, model):
        newmodel = model.copy()
        if hasattr(parent, "_meta"):  # parent is a model
            _rel = self.get_inheritance_context(appmodel, parent)
            # TODO: seems as if abstract models aren't part of models.getModels,
            #       which is why they are printed by this without any attributes.
            if _rel not in newmodel["relations"] and self.use_model(_rel["target"]):
                newmodel["relations"].append(_rel)
        return newmodel

    def sort_model_fields(self, model):
        newmodel = model.copy()
        newmodel["fields"] = sorted(
            newmodel["fields"],
            key=lambda field: (
                not field["primary_key"],
                not field["relation"],
                field["label"],
            ),
        )
        return newmodel

    def use_model(self, model_name):
        """
        Decide whether to use a model, based on the model name and the lists of
        models to exclude and include.
        """
        # Check against include list.
        if self.include_models:
            for model_pattern in self.include_models:
                model_pattern = "^%s$" % model_pattern.replace("*", ".*")
                if re.search(model_pattern, model_name):
                    return True
        # Check against exclude list.
        if self.exclude_models:
            for model_pattern in self.exclude_models:
                model_pattern = "^%s$" % model_pattern.replace("*", ".*")
                if re.search(model_pattern, model_name):
                    return False
        # Return `True` if `include_models` is falsey, otherwise return `False`.
        return not self.include_models

    def skip_field(self, field):
        if self.exclude_columns:
            if self.verbose_names and field.verbose_name:
                if field.verbose_name in self.exclude_columns:
                    return True
            if field.name in self.exclude_columns:
                return True
        if self.relation_fields_only:
            if not isinstance(
                field,
                (
                    ForeignKey,
                    ManyToManyField,
                    OneToOneField,
                    RelatedField,
                    OneToOneRel,
                    ManyToOneRel,
                ),
            ):
                return True
        return False


def generate_dot(graph_data, template="django_extensions/graph_models/digraph.dot"):
    if isinstance(template, str):
        template = loader.get_template(template)

    if not isinstance(template, Template) and not (
        hasattr(template, "template") and isinstance(template.template, Template)
    ):
        raise Exception(
            "Default Django template loader isn't used. "
            "This can lead to the incorrect template rendering. "
            "Please, check the settings."
        )

    c = Context(graph_data).flatten()
    dot = template.render(c)

    return dot


def generate_graph_data(*args, **kwargs):
    generator = ModelGraph(*args, **kwargs)
    generator.generate_graph_data()
    return generator.get_graph_data()


def use_model(model, include_models, exclude_models):
    generator = ModelGraph(
        [], include_models=include_models, exclude_models=exclude_models
    )
    return generator.use_model(model)



# -*- coding: utf-8 -*-
from django_extensions.management.jobs import WeeklyJob

from unittest import mock


WEEKLY_JOB_MOCK = mock.MagicMock()


class Job(WeeklyJob):
    help = "My sample weekly job."

    def execute(self):
        WEEKLY_JOB_MOCK()



# -*- coding: utf-8 -*-
import os

SECRET_KEY = "dummy"

TEST_RUNNER = "tests.runner.PytestTestRunner"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "django.contrib.messages",
    "django.contrib.sites",
    "tests.collisions",
    "tests.testapp",
    "tests.testapp_with_no_models_file",
    "tests.testapp_with_appconfig.apps.TestappWithAppConfigConfig",
    "django_extensions",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

DATABASES = {
    "default": {
        "ENGINE": os.environ.get(
            "DJANGO_EXTENSIONS_DATABASE_ENGINE", "django.db.backends.sqlite3"
        ),
        "NAME": os.environ.get("DJANGO_EXTENSIONS_DATABASE_NAME", ":memory:"),
        "USER": os.environ.get("DJANGO_EXTENSIONS_DATABASE_USER"),
        "PASSWORD": os.environ.get("DJANGO_EXTENSIONS_DATABASE_PASSWORD"),
        "HOST": os.environ.get("DJANGO_EXTENSIONS_DATABASE_HOST"),
        "PORT": os.environ.get("DJANGO_EXTENSIONS_DATABASE_PORT"),
    }
}

SITE_ID = 1

MEDIA_ROOT = "/tmp/django_extensions_test_media/"

MEDIA_PATH = "/media/"

ROOT_URLCONF = "tests.testapp.urls"

DEBUG = True

TEMPLATE_DEBUG = True

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": TEMPLATE_DEBUG,
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
STATIC_URL = "/static/"

SHELL_PLUS_SUBCLASSES_IMPORT_MODULES_BLACKLIST = [
    "django_extensions.mongodb.fields",
    "django_extensions.mongodb.models",
    "tests.testapp.scripts.invalid_import_script",
    "setup",
]

CACHES = {
    "default": {
        "BACKEND": "tests.management.commands.test_clear_cache.DefaultCacheMock",
    },
    "other": {
        "BACKEND": "tests.management.commands.test_clear_cache.OtherCacheMock",
    },
}

SHELL_PLUS_PRE_IMPORTS = [
    "import sys, os",
]
SHELL_PLUS_IMPORTS = [
    "from django_extensions import settings as django_extensions_settings",
]
SHELL_PLUS_POST_IMPORTS = [
    "import traceback",
    "import pprint",
    "import os as test_os",
    "from django_extensions.utils import *",
]

SILENCED_SYSTEM_CHECKS = ["models.W027", "models.W042"]



# -*- coding: utf-8 -*-
import os
import html
import shutil

from tempfile import mkdtemp

from django.template import Context, Template
from django.test import TestCase

from django_extensions.templatetags.syntax_color import generate_pygments_css


class SyntaxColorTagTests(TestCase):
    """Tests for syntax_color tags."""

    @classmethod
    def setUpClass(cls):
        cls.tmpdir = mkdtemp()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tmpdir)

    def test_should_generate_pygments_css_file_in_temp_directory(self):
        generate_pygments_css(self.tmpdir)

        self.assertTrue(os.path.exists(os.path.join(self.tmpdir, "pygments.css")))

    def test_pygments_css_should_return_highlight_css(self):
        content = """{% load syntax_color %}
{% pygments_css %}
"""
        result = Template(content).render(Context())

        self.assertIn(".highlight .hll", result)

    def test_should_colorize_with_default_lexer(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize }}
"""
        expected_result = """<div class="highlight"><pre><span></span><span class="nt">&lt;h1&gt;</span>TEST<span class="nt">&lt;/h1&gt;</span>
</pre></div>"""
        result = Template(content).render(ctx)

        self.assertHTMLEqual(result, expected_result)

    def test_colorize_should_return_value_if_lexer_class_not_found(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize:'invalid_lexer' }}
"""
        expected_result = "<h1>TEST</h1>"

        result = Template(content).render(ctx)

        self.assertHTMLEqual(html.unescape(result), expected_result)

    def test_should_colorize_table_with_default_lexer(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize_table }}
"""
        result = Template(content).render(ctx)

        self.assertIn('<table class="highlighttable">', result)
        self.assertIn('<td class="linenos">', result)
        self.assertIn(">1</", result)
        self.assertIn(
            '<span class="nt">&lt;h1&gt;</span>TEST<span class="nt">&lt;/h1&gt;</span>',
            result,
        )

    def test_colorize_table_should_return_value_if_lexer_class_not_found(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize_table:'invalid_lexer' }}
"""
        expected_result = "<h1>TEST</h1>"
        result = Template(content).render(ctx)

        self.assertHTMLEqual(html.unescape(result), expected_result)

    def test_should_colorize_noclasses_with_default_lexer(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize_noclasses }}
"""
        expected_result = """<div class="highlight" style="background: #f8f8f8"><pre style="line-height: 125%;"><span></span><span style="color: #008000; font-weight: bold">&lt;h1&gt;</span>TEST<span style="color: #008000; font-weight: bold">&lt;/h1&gt;</span>
</pre></div>"""
        result = Template(content).render(ctx)

        self.assertHTMLEqual(result, expected_result)

    def test_colorize_noclasses_should_return_value_if_lexer_class_not_found(self):
        ctx = Context({"code_string": "<h1>TEST</h1>"})
        content = """{% load syntax_color %}
{{ code_string|colorize_noclasses:'invalid_lexer' }}
"""
        expected_result = "<h1>TEST</h1>"
        result = Template(content).render(ctx)

        self.assertHTMLEqual(html.unescape(result), expected_result)



# -*- coding: utf-8 -*-
import os
import shutil

from django.core.management import call_command
from django.test import TestCase
from io import StringIO
from tests import testapp_with_no_models_file

from unittest.mock import patch


TEMPLATETAGS_DIR = os.path.join(testapp_with_no_models_file.__path__[0], "templatetags")


class CreateTemplateTagsTests(TestCase):
    """Tests for create_template_tags command."""

    def tearDown(self):
        """Remove templatetags directory after each test."""
        try:
            shutil.rmtree(TEMPLATETAGS_DIR)
        except OSError:
            pass

    def test_should_create_testapp_with_no_models_file_tags_file(self):
        call_command("create_template_tags", "testapp_with_no_models_file")
        self.assertTrue(
            os.path.isfile(
                os.path.join(TEMPLATETAGS_DIR, "testapp_with_no_models_file_tags.py")
            )
        )

    def test_should_create_custom__name_tags_file(self):
        call_command(
            "create_template_tags",
            "testapp_with_no_models_file",
            "--name",
            "custom_name_tags",
        )
        self.assertTrue(
            os.path.isfile(os.path.join(TEMPLATETAGS_DIR, "custom_name_tags.py"))
        )

    @patch("sys.stderr", new_callable=StringIO)
    @patch("shutil.copymode", side_effect=OSError)
    def test_should_print_error_notice_on_OSError(self, m_copymode, m_stderr):
        call_command("create_template_tags", "testapp_with_no_models_file")

        self.assertRegex(
            m_stderr.getvalue(),
            r"Notice: Couldn't set permission bits on \S+ You're probably using an uncommon filesystem setup. No problem.",
        )



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.template import Context, Template


class TemplateRenderingTests(TestCase):
    def setUp(self):
        self.ctx = Context(
            {
                "worldvar": "world",
            }
        )

    def test_simple_template(self):
        self.assertEqual(Template("hello world").render(self.ctx), "hello world")

    def test_simple_context(self):
        self.assertEqual(
            Template("hello {{ worldvar }}").render(self.ctx), "hello world"
        )



# -*- coding: utf-8 -*-
import ast
import shutil
import sys

from io import StringIO
from pathlib import Path

from django.core.management import call_command
from django.test import TestCase, override_settings

from .testapp.models import Name, Note, Person, Club


class DumpScriptTests(TestCase):
    def setUp(self):
        sys.stdout = StringIO()
        sys.stderr = StringIO()

    def test_runs(self):
        # lame test...does it run?
        n = Name(name="Gabriel")
        n.save()
        call_command("dumpscript", "django_extensions")
        self.assertTrue("Gabriel" in sys.stdout.getvalue())

    def test_replaced_stdout(self):
        # check if stdout can be replaced
        sys.stdout = StringIO()
        n = Name(name="Mike")
        n.save()
        tmp_out = StringIO()
        call_command("dumpscript", "django_extensions", stdout=tmp_out)
        self.assertTrue("Mike" in tmp_out.getvalue())  # script should go to tmp_out
        self.assertEqual(
            0, len(sys.stdout.getvalue())
        )  # there should not be any output to sys.stdout
        tmp_out.close()

    def test_replaced_stderr(self):
        # check if stderr can be replaced, without changing stdout
        n = Name(name="Fred")
        n.save()
        tmp_err = StringIO()
        sys.stderr = StringIO()
        call_command("dumpscript", "django_extensions", stderr=tmp_err)
        self.assertTrue(
            "Fred" in sys.stdout.getvalue()
        )  # script should still go to stdout
        self.assertTrue(
            "Name" in tmp_err.getvalue()
        )  # error output should go to tmp_err
        self.assertEqual(
            0, len(sys.stderr.getvalue())
        )  # there should not be any output to sys.stderr
        tmp_err.close()

    def test_valid_syntax(self):
        n1 = Name(name="John")
        n1.save()
        p1 = Person(name=n1, age=40)
        p1.save()
        n2 = Name(name="Jane")
        n2.save()
        p2 = Person(name=n2, age=18)
        p2.save()
        p2.children.add(p1)
        note1 = Note(note="This is the first note.")
        note1.save()
        note2 = Note(note="This is the second note.")
        note2.save()
        p2.notes.add(note1, note2)
        tmp_out = StringIO()
        call_command("dumpscript", "django_extensions", stdout=tmp_out)
        ast_syntax_tree = ast.parse(tmp_out.getvalue())
        if hasattr(ast_syntax_tree, "body"):
            self.assertTrue(len(ast_syntax_tree.body) > 1)
        else:
            self.assertTrue(len(ast_syntax_tree.asList()) > 1)
        tmp_out.close()

    @override_settings(TIME_ZONE="Asia/Seoul")
    def test_with_datetimefield(self):
        django = Club.objects.create(name="Club Django")
        Note.objects.create(
            note="Django Tips",
            club=django,
        )

        dumpscript_path = Path(__file__).parent.parent / "django_extensions" / "scripts"
        dumpscript_path.mkdir(parents=True, exist_ok=True)

        # Delete dumpscript at the end of the test
        self.addCleanup(shutil.rmtree, dumpscript_path)

        # This script will have a dateutil codes.
        # e.g. importer.locate_object(...,
        # 'date_joined': dateutil.parser.parse("2019-05-20T03:32:27.144586+09:00")
        with (dumpscript_path / "test.py").open("wt") as test:
            call_command("dumpscript", "django_extensions", stdout=test)

        # Check dumpscript without exception
        call_command("runscript", "test")

        # Check if Note is duplicated
        self.assertEqual(Note.objects.filter(note="Django Tips").count(), 2)



# -*- coding: utf-8 -*-
"""
URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/

Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""

from django.urls import path
from django.contrib import admin
from django.contrib.auth import views as auth_views

login_view = (
    auth_views.LoginView.as_view()
    if hasattr(auth_views, "LoginView")
    else auth_views.login
)
logout_view = (
    auth_views.LogoutView.as_view()
    if hasattr(auth_views, "LogoutView")
    else auth_views.logout
)

urlpatterns = [
    path("login/", login_view, {"template_name": "login.html"}, name="login"),
    path("logout/", logout_view, name="logout"),
    path("admin/", admin.site.urls),
]



# -*- coding: utf-8 -*-
from io import StringIO

from django.core.management import call_command, CommandError
from django.contrib.auth.models import User

from django_extensions.management.commands.set_fake_emails import Command

import pytest


@pytest.fixture(scope="module")
def django_db_setup(django_db_setup, django_db_blocker):
    """Load to database a set of users, create for export
    emails command testing"""
    with django_db_blocker.unblock():
        call_command("loaddata", "group.json")
        call_command("loaddata", "user.json")


@pytest.mark.django_db()
def test_without_args(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email)
    out, err = capsys.readouterr()
    assert "Changed 3 emails" in out

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)


@pytest.mark.django_db()
def test_no_admin(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "-a")
    out, err = capsys.readouterr()
    assert "Changed 2 emails" in out

    emails = User.objects.filter(is_superuser=False).values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)

    emails = User.objects.filter(is_superuser=True).values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)


@pytest.mark.django_db()
def test_no_staff(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "-s")
    out, err = capsys.readouterr()
    assert "Changed 2 emails" in out

    emails = User.objects.filter(is_staff=False).values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)

    emails = User.objects.filter(is_staff=True).values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)


@pytest.mark.django_db()
def test_include_groups(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "--include-groups=Attendees")
    out, err = capsys.readouterr()
    assert "Changed 2 emails" in out

    emails = User.objects.filter(is_superuser=False).values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)

    emails = User.objects.filter(is_superuser=True).values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)


@pytest.mark.django_db()
def test_include_groups_which_does_not_exists(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    with pytest.raises(CommandError, match="No groups matches filter: TEST"):
        call_command("set_fake_emails", "--include-groups=TEST")

    assert not User.objects.filter(email__endswith="@example.com").exists()


@pytest.mark.django_db()
def test_exclude_groups(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "--exclude-groups=Attendees")
    out, err = capsys.readouterr()
    assert "Changed 1 emails" in out

    emails = User.objects.filter(is_superuser=False).values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    emails = User.objects.filter(is_superuser=True).values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)


@pytest.mark.django_db()
def test_exclude_groups_which_does_not_exists(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    with pytest.raises(CommandError, match="No groups matches filter: TEST"):
        call_command("set_fake_emails", "--exclude-groups=TEST")

    assert not User.objects.filter(email__endswith="@example.com").exists()


@pytest.mark.django_db()
def test_include_regexp(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "--include=.*briel")
    out, err = capsys.readouterr()
    assert "Changed 1 emails" in out

    emails = User.objects.exclude(username="Gabriel").values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    emails = User.objects.filter(username="Gabriel").values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)


@pytest.mark.django_db()
def test_exclude_regexp(capsys, settings):
    settings.DEBUG = True

    emails = User.objects.values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    generate_email = Command()
    call_command(generate_email, "--exclude=.*briel")
    out, err = capsys.readouterr()
    assert "Changed 2 emails" in out

    emails = User.objects.filter(username="Gabriel").values_list("email", flat=True)
    assert all(email.endswith("@gmail.com") for email in emails)

    emails = User.objects.exclude(username="Gabriel").values_list("email", flat=True)
    assert all(email.endswith("@example.com") for email in emails)


def test_without_debug(settings):
    settings.DEBUG = False

    out = StringIO()
    with pytest.raises(CommandError, match="Only available in debug mode"):
        call_command("set_fake_emails", verbosity=3, stdout=out, stderr=out)



# -*- coding: utf-8 -*-
import os
import shutil

from django.conf import settings
from django.core.management import CommandError, call_command
from django.test import TestCase
from django.test.utils import override_settings
from io import StringIO

from django_extensions.management.commands.sync_s3 import Command

from unittest.mock import Mock, patch


class SyncS3TestsMixin:
    def setUp(self):
        self.m_boto = Mock()
        self.boto_patch = patch(
            "django_extensions.management.commands.sync_s3.boto",
            create=True,
            boto=self.m_boto,
        )
        self.boto_patch.start()

    def tearDown(self):
        super().tearDown()
        self.boto_patch.stop()


@patch("django_extensions.management.commands.sync_s3.HAS_BOTO", True)
class SyncS3ExceptionsTests(SyncS3TestsMixin, TestCase):
    def test_should_raise_ImportError_if_boto_is_not_installed(self):
        with patch("django_extensions.management.commands.sync_s3.HAS_BOTO", False):
            with self.assertRaisesRegex(
                CommandError,
                r"Please install the 'boto' Python library. \(\$ pip install boto\)",
            ):
                call_command("sync_s3")

    @override_settings(AWS_ACCESS_KEY_ID=None)
    def test_should_raise_CommandError_if_AWS_ACCESS_KEY_ID_is_not_set_or_is_set_to_None(
        self,
    ):
        with self.assertRaisesRegex(
            CommandError,
            "Missing AWS keys from settings file. Please supply both AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY",
        ):
            call_command("sync_s3")

    @override_settings(AWS_SECRET_ACCESS_KEY=None)
    def test_should_raise_CommandError_if_AWS_SECRET_ACCESS_KEY_is_not_set_or_is_set_to_None(
        self,
    ):
        with self.assertRaisesRegex(
            CommandError,
            "Missing AWS keys from settings file. Please supply both AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY",
        ):
            call_command("sync_s3")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="",
    )
    def test_should_raise_CommandError_if_AWS_BUCKET_NAME_is_set_to_None(self):
        with self.assertRaisesRegex(CommandError, "AWS_BUCKET_NAME cannot be empty"):
            call_command("sync_s3")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
    )
    def test_should_raise_CommandError_if_AWS_BUCKET_NAME_does_not_have_AWS_BUCKET_NAME_attr(
        self,
    ):
        del settings.AWS_BUCKET_NAME

        with self.assertRaisesRegex(
            CommandError,
            "Missing bucket name from settings file. Please add the AWS_BUCKET_NAME to your settings file.",
        ):
            call_command("sync_s3")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="bucket_name",
        MEDIA_ROOT=None,
    )
    def test_should_raise_CommandError_if_MEDIA_ROOT_is_None(self):
        with self.assertRaisesRegex(
            CommandError, "MEDIA_ROOT must be set in your settings."
        ):
            call_command("sync_s3")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="bucket_name",
    )
    def test_should_raise_CommandError_if_settings_does_not_have_MEDIA_ROOT_attr(self):
        del settings.MEDIA_ROOT

        with self.assertRaisesRegex(
            CommandError, "MEDIA_ROOT must be set in your settings."
        ):
            call_command("sync_s3")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="bucket_name",
    )
    def test_should_raise_CommandError_when_media_only_and_static_only_options_together(
        self,
    ):
        with self.assertRaisesRegex(
            CommandError,
            "Can't use --media-only and --static-only together. Better not use anything...",
        ):
            call_command("sync_s3", "--media-only", "--static-only")

    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="bucket_name",
    )
    def test_should_raise_CommandError_when_medi(self):
        m_bucket = Mock()
        del settings.AWS_CLOUDFRONT_DISTRIBUTION

        self.m_boto.connect_s3.return_value.get_bucket.return_value = m_bucket
        self.m_boto.s3.key.Key.return_value = "bucket_key"

        with self.assertRaisesRegex(
            CommandError,
            "An object invalidation was requested but the variable AWS_CLOUDFRONT_DISTRIBUTION is not present in your settings.",
        ):
            call_command("sync_s3", "--media-only", "--invalidate")


@patch("django_extensions.management.commands.sync_s3.HAS_BOTO", Mock(side_effect=True))
class SyncS3Tests(TestCase):
    def setUp(self):
        current_dir = os.path.dirname(os.path.abspath(__file__))
        media_root = os.path.join(current_dir, "media")
        if os.path.isdir(media_root):
            shutil.rmtree(media_root)
        os.mkdir(media_root)
        test_dirs = [
            os.path.join(media_root, "testdir1"),
            os.path.join(media_root, "testdir2"),
            os.path.join(media_root, "testdir3"),
            os.path.join(media_root, "testdir4"),
            os.path.join(media_root, "testsamenamedir"),
        ]
        for dir_ in test_dirs:
            os.mkdir(dir_)

        test_sub_dir = os.path.join(media_root, "testdir2", "testsubdir1")
        os.mkdir(test_sub_dir)

        test_sub_dir_base = os.path.join(media_root, "testdir1")
        test_sub_dirs = [
            os.path.join(test_sub_dir_base, "testsubdir1"),
            os.path.join(test_sub_dir_base, "testsubdir2"),
            os.path.join(test_sub_dir_base, "testsubdir3"),
            os.path.join(test_sub_dir_base, "testsubdir4"),
        ]

        for dir_ in test_sub_dirs:
            os.mkdir(dir_)

        test_sub_dir_base2 = os.path.join(media_root, "testdir3")
        test_sub_dirs_2 = [
            os.path.join(test_sub_dir_base2, "testsubdir1"),
            os.path.join(test_sub_dir_base2, "testsubdir2"),
            os.path.join(test_sub_dir_base2, "testsubdir3"),
            os.path.join(test_sub_dir_base2, "testsubdir4"),
            os.path.join(test_sub_dir_base2, "testsamenamedir"),
        ]

        for dir_ in test_sub_dirs_2:
            os.mkdir(dir_)

    def tearDown(self):
        current_dir = os.path.dirname(os.path.abspath(__file__))
        media_root = os.path.join(current_dir, "media")
        shutil.rmtree(media_root)

    @override_settings(
        MEDIA_ROOT=os.path.join(os.path.dirname(os.path.abspath(__file__)), "media")
    )
    @override_settings(
        STATIC_ROOT=os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")
    )
    @override_settings(AWS_ACCESS_KEY_ID="FAKE_KEY")
    @override_settings(AWS_SECRET_ACCESS_KEY="FAKE_SECRET_KEY")
    @override_settings(AWS_BUCKET_NAME="FAKE_BUCKET_NAME")
    def test_sync_s3_dir_exclusions(self):
        sync_s3_command = Command()
        try:
            call_command(
                "sync_s3",
                "--acl=authenticated-read",
                "--filter-list=testsamenamedir,testdir1",
            )
        except Exception:
            # Exception is expected, we're not actually attempting to connect to S3
            self.assertEqual(0, 0)

        for directory in sync_s3_command.DIRECTORIES:
            for root, dirs, files in os.walk(directory):
                sync_s3_command.upload_s3(
                    ("FAKE_BUCKET", "FAKE_KEY", "FAKE_BUCKET_NAME", directory),
                    root,
                    files,
                    dirs,
                )
                dir_name = os.path.basename(root)
                if dir_name == "testdir1":
                    self.assertEqual(len(dirs), 0)
                elif dir_name == "testdir2":
                    self.assertEqual(len(dirs), 1)
                elif dir_name == "testdir3":
                    self.assertEqual(len(dirs), 5)


@patch("django_extensions.management.commands.sync_s3.HAS_BOTO", Mock(side_effect=True))
class SyncS3CommandTests(SyncS3TestsMixin, TestCase):
    @override_settings(
        AWS_ACCESS_KEY_ID="access_key_id",
        AWS_SECRET_ACCESS_KEY="secret_access_key",
        AWS_BUCKET_NAME="bucket_name",
    )
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_raise_CommandError_when_medi(self, m_stdout):
        self.m_boto.connect_s3.return_value.get_bucket.return_value = Mock()
        self.m_boto.s3.key.Key.return_value = "bucket_key"

        call_command("sync_s3", "--media-only")

        self.assertIn("0 files uploaded.", m_stdout.getvalue())
        self.assertIn("0 files skipped.", m_stdout.getvalue())



# -*- coding: utf-8 -*-
"""
Daily cleanup job.

Can be run as a cronjob to clean out old data from the database (only expired
sessions at the moment).
"""

from django.conf import settings
from django.core.cache import caches

from django_extensions.management.jobs import DailyJob


class Job(DailyJob):
    help = "Cache (db) cleanup Job"

    def execute(self):
        if hasattr(settings, "CACHES"):
            for cache_name, cache_options in settings.CACHES.items():
                if cache_options["BACKEND"].endswith("DatabaseCache"):
                    cache = caches[cache_name]
                    cache.clear()
            return



# -*- coding: utf-8 -*-
from unittest import mock
import pytest
from io import StringIO

from django.conf import settings
from django.apps import apps
from django.test import TestCase

# from django.core.management import call_command
from django_extensions.management.commands.sqldiff import (
    SqliteSQLDiff,
    Command,
    MySQLDiff,
    PostgresqlSQLDiff,
)
from tests.testapp.models import (
    PostWithUniqField,
    SluggedWithUniqueTogetherTestModel,
    RandomCharTestModelUniqueTogether,
    SqlDiffUniqueTogether,
    SqlDiff,
    SqlDiffIndexes,
)


class SqlDiffTests(TestCase):
    def setUp(self):
        self.parser = Command().create_parser("test", "sqldiff")
        self.args = ["-a"]
        self.options = self.parser.parse_args(args=self.args)
        self.tmp_out = StringIO()
        self.tmp_err = StringIO()

    def _include_proxy_models_testing(self, should_include_proxy_models):  # type: (bool) -> ()
        if should_include_proxy_models:
            self.args.append("--include-proxy-models")
        self.options = self.parser.parse_args(args=self.args)
        instance = SqliteSQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        instance.load()
        instance.find_differences()
        checked_models = {
            "%s.%s" % (app_label, model_name)
            for app_label, model_name, _ in instance.differences
        }
        self.assertEqual(
            should_include_proxy_models,
            "testapp.PostWithTitleOrdering" in checked_models,
        )

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.sqlite3",
        reason="Test can only run on sqlite3",
    )
    def test_sql_diff_without_proxy_models(self):
        self._include_proxy_models_testing(False)

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.sqlite3",
        reason="Test can only run on sqlite3",
    )
    def test_sql_diff_with_proxy_models(self):
        self._include_proxy_models_testing(True)

    def test_format_field_names(self):
        instance = MySQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        expected_field_name = ["name", "email", "address"]
        self.assertEqual(
            instance.format_field_names(["Name", "EMAIL", "aDDress"]),
            expected_field_name,
        )

    def test_get_index_together(self):
        instance = MySQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        self.assertEqual(
            instance.get_index_together(SqlDiff._meta), [("number", "creator")]
        )
        self.assertEqual(
            instance.get_index_together(SqlDiffIndexes._meta), [("first", "second")]
        )

    def test_get_unique_together(self):
        instance = MySQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        self.assertEqual(
            instance.get_unique_together(SluggedWithUniqueTogetherTestModel._meta),
            [("slug", "category")],
        )
        self.assertEqual(
            instance.get_unique_together(RandomCharTestModelUniqueTogether._meta),
            [("random_char_field", "common_field")],
        )
        self.assertEqual(
            instance.get_unique_together(SqlDiffUniqueTogether._meta), [("aaa", "bbb")]
        )
        self.assertEqual(
            instance.get_unique_together(PostWithUniqField._meta),
            [("common_field", "uniq_field")],
        )

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.mysql",
        reason="Test can only run on mysql",
    )
    def test_mysql_to_dict(self):
        mysql_instance = MySQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        mysql_dict = mysql_instance.sql_to_dict(
            """select 1 as "foo", 1 + 1 as "BAR";""", []
        )
        self.assertEqual(mysql_dict, [{"bar": 2, "foo": 1}])

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.mysql",
        reason="Test can only run on mysql",
    )
    @mock.patch(
        "django_extensions.management.commands.sqldiff.MySQLDiff.format_field_names"
    )
    def test_invalid_mysql_to_dict(self, format_field_names):
        format_field_names.side_effect = lambda x: x
        mysql_instance = MySQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )
        mysql_dict = mysql_instance.sql_to_dict(
            """select 1 as "foo", 1 + 1 as "BAR";""", []
        )
        self.assertNotEquals(mysql_dict, [{"bar": 2, "foo": 1}])

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.sqlite3",
        reason="Test can only run on sqlite3",
    )
    def test_sqlite_to_dict(self):
        sqlite_instance = SqliteSQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )

        sqlite_dict = sqlite_instance.sql_to_dict(
            """select 1 as "foo", 1 + 1 as "BAR";""", []
        )
        self.assertEqual(sqlite_dict, [{"BAR": 2, "foo": 1}])

    @pytest.mark.skipif(
        settings.DATABASES["default"]["ENGINE"] != "django.db.backends.postgresql",
        reason="Test can only run on postgresql",
    )
    def test_postgresql_to_dict(self):
        postgresql_instance = PostgresqlSQLDiff(
            apps.get_models(include_auto_created=True),
            vars(self.options),
            stdout=self.tmp_out,
            stderr=self.tmp_err,
        )

        postgresql_dict = postgresql_instance.sql_to_dict(
            """select 1 as "foo", 1 + 1 as "BAR";""", []
        )
        self.assertEqual(postgresql_dict, [{"BAR": 2, "foo": 1}])



# -*- coding: utf-8 -*-
from io import StringIO

from django.core.management import CommandError, call_command
from django.test import TestCase
from django.test.utils import override_settings

from unittest.mock import patch


MYSQL_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.mysql",
    "NAME": "dbatabase",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "127.0.0.1",
    "PORT": "3306",
}

SQLITE3_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.sqlite3",
    "NAME": "db.sqlite3",
}

POSTGRESQL_DATABASE_SETTINGS = {
    "ENGINE": "django.db.backends.postgresql",
    "NAME": "database",
    "USER": "foo",
    "PASSWORD": "bar",
    "HOST": "localhost",
    "PORT": "5432",
}

POSTGRESQL_DATABASE_SETTINGS_SOCKET_MODE = {
    "ENGINE": "django.db.backends.postgresql",
    "NAME": "database",
    "USER": "",
    "PASSWORD": "",
    "HOST": "",
    "PORT": "",
}


class SqlcreateExceptionsTests(TestCase):
    """Test for sqlcreate exception."""

    def test_should_raise_CommandError_if_database_is_unknown(self):
        with self.assertRaisesRegex(CommandError, "Unknown database unknown"):
            call_command("sqlcreate", "--database=unknown")


class SqlCreateTests(TestCase):
    """Tests for sqlcreate command."""

    @override_settings(DATABASES={"default": MYSQL_DATABASE_SETTINGS})
    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    @patch("django_extensions.management.commands.sqlcreate.socket")
    def test_should_print_SQL_create_database_statement_for_mysql(
        self, m_socket, m_stdout, m_stderr
    ):
        m_socket.gethostname.return_value = "tumbleweed"
        expected_error = """-- WARNING!: https://docs.djangoproject.com/en/dev/ref/databases/#collation-settings
-- Please read this carefully! Collation will be set to utf8_bin to have case-sensitive data.
"""
        expected_statement = """CREATE DATABASE dbatabase CHARACTER SET utf8 COLLATE utf8_bin;
GRANT ALL PRIVILEGES ON dbatabase.* to 'foo'@'tumbleweed' identified by 'bar';
"""

        call_command("sqlcreate")

        self.assertEqual(expected_statement, m_stdout.getvalue())
        self.assertEqual(expected_error, m_stderr.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_SQL_create_database_statement_for_postgresql(self, m_stdout):
        expected_statement = """CREATE USER foo WITH ENCRYPTED PASSWORD 'bar' CREATEDB;
CREATE DATABASE database WITH ENCODING 'UTF-8' OWNER "foo";
GRANT ALL PRIVILEGES ON DATABASE database TO foo;
"""

        call_command("sqlcreate")

        self.assertEqual(expected_statement, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_DATABASE_SETTINGS_SOCKET_MODE})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_SQL_create_database_statement_only_for_postgresql_when_unix_domain_socket_mode_is_used(
        self, m_stdout
    ):
        expected_statement = """-- Assuming that unix domain socket connection mode is being used because
-- USER or PASSWORD are blank in Django DATABASES configuration.
CREATE DATABASE database WITH ENCODING 'UTF-8';
"""

        call_command("sqlcreate")

        self.assertEqual(expected_statement, m_stdout.getvalue())

    @override_settings(DATABASES={"default": POSTGRESQL_DATABASE_SETTINGS})
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_SQL_drop_and_create_database_statement_for_postgresql(
        self, m_stdout
    ):
        expected_statement = """DROP DATABASE IF EXISTS database;
DROP USER IF EXISTS foo;
CREATE USER foo WITH ENCRYPTED PASSWORD 'bar' CREATEDB;
CREATE DATABASE database WITH ENCODING 'UTF-8' OWNER "foo";
GRANT ALL PRIVILEGES ON DATABASE database TO foo;
"""

        call_command("sqlcreate", "--drop")

        self.assertEqual(expected_statement, m_stdout.getvalue())

    @override_settings(DATABASES={"default": SQLITE3_DATABASE_SETTINGS})
    @patch("sys.stderr", new_callable=StringIO)
    def test_should_print_stderr_for_sqlite3(self, m_stderr):
        expected_error = (
            "-- manage.py migrate will automatically create a sqlite3 database file.\n"
        )

        call_command("sqlcreate")

        self.assertEqual(expected_error, m_stderr.getvalue())

    @override_settings(
        DATABASES={
            "unknown": {
                "ENGINE": "django.db.backends.unknown",
                "NAME": "database",
                "USER": "foo",
            }
        }
    )
    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_print_stderr_and_standard_create_database_statement_for_unsupported_engine(
        self, m_stdout, m_stderr
    ):
        expected_error = "-- Don't know how to handle 'django.db.backends.unknown' falling back to SQL.\n"
        expected_statement = """CREATE DATABASE database;
GRANT ALL PRIVILEGES ON DATABASE database to foo;
"""

        call_command("sqlcreate", "--database=unknown")

        self.assertEqual(expected_error, m_stderr.getvalue())
        self.assertEqual(expected_statement, m_stdout.getvalue())



# -*- coding: utf-8 -*-
import threading

from django.core.handlers.wsgi import WSGIHandler

tld = threading.local()
tld.wsgi_tb = None


def null_technical_500_response(request, exc_type, exc_value, tb, status_code=500):
    """
    Alternative function for django.views.debug.technical_500_response.

    Django's convert_exception_to_response() wrapper is called on each 'Middleware' object to avoid
    leaking exceptions. If an uncaught exception is raised, the wrapper calls technical_500_response()
    to create a response for django's debug view.

    Runserver_plus overrides the django debug view's technical_500_response() function to allow for
    an enhanced WSGI debugger view to be displayed. However, because Django calls
    convert_exception_to_response() on each object in the stack of Middleware objects, re-raising an
    error quickly pollutes the traceback displayed.

    Runserver_plus only needs needs traceback frames relevant to WSGIHandler Middleware objects, so
    only store the traceback if it is for a WSGIHandler. If an exception is not raised here, Django
    eventually throws an error for not getting a valid response object for its debug view.
    """  # noqa: E501
    try:
        # Store the most recent tb for WSGI requests.
        # The class can be found in the second frame of the tb
        if isinstance(tb.tb_next.tb_frame.f_locals.get("self"), WSGIHandler):
            tld.wsgi_tb = tb
        elif tld.wsgi_tb:
            tb = tld.wsgi_tb
    except AttributeError:
        pass

    try:
        if exc_value is None:
            exc_value = exc_type()
        if exc_value.__traceback__ is not tb:
            raise exc_value.with_traceback(tb)
        raise exc_value
    finally:
        exc_value = None
        tb = None



# -*- coding: utf-8 -*-
from django_extensions.management.jobs import BaseJob


class Job(BaseJob):
    help = "My sample job."

    def execute(self):
        # executing empty sample job
        pass



# -*- coding: utf-8 -*-
import time

from django.test import TestCase
from django.test.utils import override_settings

from django_extensions.logging.filters import RateLimiterFilter

from unittest import mock


TEST_SUBJECT = "test_subect"


class RateLimiterFilterTests(TestCase):
    """Tests for RateLimiterFilter."""

    def setUp(self):
        self.rate_limiter_filter = RateLimiterFilter()
        self.record = mock.Mock(msg=TEST_SUBJECT)
        self.record.getMessage.return_value = TEST_SUBJECT.encode()
        self.time_patch = mock.patch.object(time, "time", return_value="test_time")
        self.time_patch.start()

    def tearDown(self):
        self.time_patch.stop()

    @override_settings(RATE_LIMITER_FILTER_PREFIX="test_prefix")
    @mock.patch("django.core.cache.cache")
    def test_should_incr_cache_with_custom_prefix_and_return_False(self, m_cache):
        m_cache.get_many.return_value = {
            "test_prefix:114392702498ad1d75c1829b9519b8c7": 10,
            "test_prefix:114392702498ad1d75c1829b9519b8c7:count": 1,
        }

        result = self.rate_limiter_filter.filter(self.record)

        self.assertIs(m_cache.set.called, False)
        m_cache.incr.assert_called_once_with(
            "test_prefix:114392702498ad1d75c1829b9519b8c7:count"
        )
        self.assertIs(result, False)

    @override_settings(RATE_LIMITER_FILTER_RATE=1)
    @mock.patch("django.core.cache.cache")
    def test_should_set_cache_key_with_custom_rate_and_return_True(self, m_cache):
        m_cache.get_many.return_value = {}
        expected_calls = [
            mock.call(
                "ratelimiterfilter:114392702498ad1d75c1829b9519b8c7:count", 1, 61
            ),
            mock.call(
                "ratelimiterfilter:114392702498ad1d75c1829b9519b8c7", "test_time", 1
            ),
        ]

        result = self.rate_limiter_filter.filter(self.record)

        self.assertEqual(self.record.msg, "[1x] test_subect")
        m_cache.set.assert_has_calls(expected_calls, any_order=False)
        self.assertIs(result, True)

    @mock.patch("django.core.cache.cache")
    def test_should_modify_record_msg_and_return_True(self, m_cache):
        """Default rate and prefix values."""
        m_cache.get_many.return_value = {
            "ratelimiterfilter:114392702498ad1d75c1829b9519b8c7:count": 999,
        }

        result = self.rate_limiter_filter.filter(self.record)

        self.assertEqual(self.record.msg, "[999x] test_subect")
        m_cache.set.assert_called_once_with(
            "ratelimiterfilter:114392702498ad1d75c1829b9519b8c7", "test_time", 10
        )
        self.assertIs(result, True)



# -*- coding: utf-8 -*-
from io import StringIO

from django.core.management import call_command
from django.test import TestCase

from unittest.mock import patch


class GenerateSecretKeyTests(TestCase):
    """Tests for generate_secret_key command."""

    @patch(
        "django_extensions.management.commands.generate_secret_key.get_random_secret_key"
    )
    @patch("sys.stdout", new_callable=StringIO)
    def test_should_return_random_secret_key(self, m_stdout, m_get_random_secret):
        m_get_random_secret.return_value = "random_secret_key"

        call_command("generate_secret_key")

        self.assertIn("random_secret_key", m_stdout.getvalue())



# -*- coding: utf-8 -*-
from django_extensions.management.email_notifications import EmailNotificationCommand


class Command(EmailNotificationCommand):
    help = "Just for email_notifications testing purpose"

    def handle(self, *args, **kwargs):
        raise Exception()



# -*- coding: utf-8 -*-
import pytest

from os.path import join

from django_extensions.management.commands.runserver_plus import (
    Command as RunServerCommand,
)

from unittest import mock


location = join("some", "strange", "path")
different_path = join("some", "other", "path")


@pytest.mark.parametrize(
    "cert_option, key_file_option, expected_cert_path, expected_key_path",
    [
        ["hello", None, join(location, "hello.crt"), join(location, "hello.key")],
        ["hello.crt", None, join(location, "hello.crt"), join(location, "hello.key")],
        [None, "hello", join(location, "hello.crt"), join(location, "hello.key")],
        [None, "hello.key", join(location, "hello.crt"), join(location, "hello.key")],
        ["first", "second", join(location, "first.crt"), join(location, "second.key")],
        [
            "first.pem",
            "second.pem",
            join(location, "first.pem"),
            join(location, "second.pem"),
        ],
        ["cert.pem", "key.pem", join(location, "cert.pem"), join(location, "key.pem")],
        [
            join(location, "hello"),
            None,
            join(location, "hello.crt"),
            join(location, "hello.key"),
        ],
        [
            None,
            join(location, "hello"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
        ],
        [
            join(location, "hello"),
            join(location, "hello"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
        ],
        [
            join(location, "hello.crt"),
            join(location, "hello.key"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
        ],
        [
            join(location, "hello.key"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
            join(location, "hello.crt"),
        ],
        [
            join(location, "cert.pem"),
            join(location, "key.pem"),
            join(location, "cert.pem"),
            join(location, "key.pem"),
        ],
        [
            join(location, "hello.crt"),
            join(location, "hello.key"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
        ],
        [
            join(location, "other"),
            join(location, "hello.key"),
            join(location, "other.crt"),
            join(location, "hello.key"),
        ],
        [
            join(location, "hello.key"),
            join(location, "hello.crt"),
            join(location, "hello.key"),
            join(location, "hello.crt"),
        ],
        [
            join(different_path, "hello"),
            None,
            join(different_path, "hello.crt"),
            join(different_path, "hello.key"),
        ],
        [
            None,
            join(different_path, "hello"),
            join(different_path, "hello.crt"),
            join(different_path, "hello.key"),
        ],
        [
            join(location, "hello.crt"),
            join(different_path, "hello.key"),
            join(location, "hello.crt"),
            join(different_path, "hello.key"),
        ],
        [
            join(different_path, "hello.crt"),
            join(location, "hello.key"),
            join(different_path, "hello.crt"),
            join(location, "hello.key"),
        ],
    ],
)
def test_determining_paths(
    cert_option, key_file_option, expected_cert_path, expected_key_path
):
    with mock.patch(
        "django_extensions.management.commands.runserver_plus.os.getcwd",
        return_value=location,
    ):
        options = {"cert_path": cert_option, "key_file_path": key_file_option}
        result_cert_path, result_key_path = RunServerCommand.determine_ssl_files_paths(
            options
        )
        assert expected_cert_path == result_cert_path
        assert expected_key_path == result_key_path



# -*- coding: utf-8 -*-
"""
Author Igor Támara igor@tamarapatino.org
Use this little program as you wish, if you
include it in your work, let others know you
are using it preserving this note, you have
the right to make derivative works, Use it
at your own risk.
Tested to work on(etch testing 13-08-2007):
  Python 2.4.4 (#2, Jul 17 2007, 11:56:54)
  [GCC 4.1.3 20070629 (prerelease) (Debian 4.1.2-13)] on linux2
"""

import codecs
import gzip
import re
import sys
from xml.dom.minidom import Node, parseString

dependclasses = ["User", "Group", "Permission", "Message"]

# Type dictionary translation types SQL -> Django
tsd = {
    "text": "TextField",
    "date": "DateField",
    "varchar": "CharField",
    "int": "IntegerField",
    "float": "FloatField",
    "serial": "AutoField",
    "boolean": "BooleanField",
    "numeric": "FloatField",
    "timestamp": "DateTimeField",
    "bigint": "IntegerField",
    "datetime": "DateTimeField",
    "time": "TimeField",
    "bool": "BooleanField",
}

# convert varchar -> CharField
v2c = re.compile(r"varchar\((\d+)\)")


def find_index(fks, id_):
    """
    Look for the id on fks, fks is an array of arrays, each array has on [1]
    the id of the class in a dia diagram.  When not present returns None, else
    it returns the position of the class with id on fks
    """
    for i, _ in fks.items():
        if fks[i][1] == id_:
            return i
    return None


def addparentstofks(rels, fks):
    """
    Get a list of relations, between parents and sons and a dict of
    clases named in dia, and modifies the fks to add the parent as fk to get
    order on the output of classes and replaces the base class of the son, to
    put the class parent name.
    """
    for j in rels:
        son = find_index(fks, j[1])
        parent = find_index(fks, j[0])
        fks[son][2] = fks[son][2].replace("models.Model", parent)
        if parent not in fks[son][0]:
            fks[son][0].append(parent)


def dia2django(archivo):
    models_txt = ""
    f = codecs.open(archivo, "rb")
    # dia files are gzipped
    data = gzip.GzipFile(fileobj=f).read()
    ppal = parseString(data)
    # diagram -> layer -> object -> UML - Class -> name,
    #   (attribs : composite -> name,type)
    datos = (
        ppal.getElementsByTagName("dia:diagram")[0]
        .getElementsByTagName("dia:layer")[0]
        .getElementsByTagName("dia:object")
    )
    clases = {}
    herit = []
    imports = str("")
    for i in datos:
        # Look for the classes
        if i.getAttribute("type") == "UML - Class":
            myid = i.getAttribute("id")
            for j in i.childNodes:
                if j.nodeType == Node.ELEMENT_NODE and j.hasAttributes():
                    if j.getAttribute("name") == "name":
                        actclas = (
                            j.getElementsByTagName("dia:string")[0]
                            .childNodes[0]
                            .data[1:-1]
                        )
                        myname = "\nclass %s(models.Model) :\n" % actclas
                        clases[actclas] = [[], myid, myname, 0]
                    if j.getAttribute("name") == "attributes":
                        for ll in j.getElementsByTagName("dia:composite"):
                            if ll.getAttribute("type") == "umlattribute":
                                # Look for the attribute name and type
                                for k in ll.getElementsByTagName("dia:attribute"):
                                    if k.getAttribute("name") == "name":
                                        nc = (
                                            k.getElementsByTagName("dia:string")[0]
                                            .childNodes[0]
                                            .data[1:-1]
                                        )
                                    elif k.getAttribute("name") == "type":
                                        tc = (
                                            k.getElementsByTagName("dia:string")[0]
                                            .childNodes[0]
                                            .data[1:-1]
                                        )
                                    elif k.getAttribute("name") == "value":
                                        val = (
                                            k.getElementsByTagName("dia:string")[0]
                                            .childNodes[0]
                                            .data[1:-1]
                                        )
                                        if val == "##":
                                            val = ""
                                    elif (
                                        k.getAttribute("name") == "visibility"
                                        and k.getElementsByTagName("dia:enum")[
                                            0
                                        ].getAttribute("val")
                                        == "2"
                                    ):
                                        if (
                                            tc.replace(" ", "")
                                            .lower()
                                            .startswith("manytomanyfield(")
                                        ):
                                            # If we find a class not in our model that
                                            # is marked as being to another model
                                            newc = tc.replace(" ", "")[16:-1]
                                            if dependclasses.count(newc) == 0:
                                                dependclasses.append(newc)
                                        if (
                                            tc.replace(" ", "")
                                            .lower()
                                            .startswith("foreignkey(")
                                        ):
                                            # If we find a class not in our model that
                                            # is marked as being to another model
                                            newc = tc.replace(" ", "")[11:-1]
                                            if dependclasses.count(newc) == 0:
                                                dependclasses.append(newc)

                                # Mapping SQL types to Django
                                varch = v2c.search(tc)
                                if tc.replace(" ", "").startswith("ManyToManyField("):
                                    myfor = tc.replace(" ", "")[16:-1]
                                    if actclas == myfor:
                                        # In case of a recursive type, we use 'self'
                                        tc = tc.replace(myfor, "'self'")
                                    elif clases[actclas][0].count(myfor) == 0:
                                        # Adding related class
                                        if myfor not in dependclasses:
                                            # In case we are using Auth classes or
                                            # external via protected dia visibility
                                            clases[actclas][0].append(myfor)
                                    tc = "models." + tc
                                    if len(val) > 0:
                                        tc = tc.replace(")", "," + val + ")")
                                elif tc.find("Field") != -1:
                                    if tc.count("()") > 0 and len(val) > 0:
                                        tc = "models.%s" % tc.replace(
                                            ")", "," + val + ")"
                                        )
                                    else:
                                        tc = "models.%s(%s)" % (tc, val)
                                elif tc.replace(" ", "").startswith("ForeignKey("):
                                    myfor = tc.replace(" ", "")[11:-1]
                                    if actclas == myfor:
                                        # In case of a recursive type, we use 'self'
                                        tc = tc.replace(myfor, "'self'")
                                    elif clases[actclas][0].count(myfor) == 0:
                                        # Adding foreign classes
                                        if myfor not in dependclasses:
                                            # In case we are using Auth classes
                                            clases[actclas][0].append(myfor)
                                    tc = "models." + tc
                                    if len(val) > 0:
                                        tc = tc.replace(")", "," + val + ")")
                                elif varch is None:
                                    tc = (
                                        "models."
                                        + tsd[tc.strip().lower()]
                                        + "("
                                        + val
                                        + ")"
                                    )
                                else:
                                    tc = (
                                        "models.CharField(max_length="
                                        + varch.group(1)
                                        + ")"
                                    )
                                    if len(val) > 0:
                                        tc = tc.replace(")", ", " + val + " )")
                                if not (nc == "id" and tc == "AutoField()"):
                                    clases[actclas][2] += "    %s = %s\n" % (nc, tc)
        elif i.getAttribute("type") == "UML - Generalization":
            mycons = ["A", "A"]
            a = i.getElementsByTagName("dia:connection")
            for j in a:
                if len(j.getAttribute("to")):
                    mycons[int(j.getAttribute("handle"))] = j.getAttribute("to")
            print(mycons)
            if "A" not in mycons:
                herit.append(mycons)
        elif i.getAttribute("type") == "UML - SmallPackage":
            a = i.getElementsByTagName("dia:string")
            for j in a:
                if len(j.childNodes[0].data[1:-1]):
                    imports += str(
                        "from %s.models import *" % j.childNodes[0].data[1:-1]
                    )

    addparentstofks(herit, clases)
    # Ordering the appearance of classes
    # First we make a list of the classes each classs is related to.
    ordered = []
    for j, k in clases.items():
        k[2] += '\n    def __str__(self):\n        return u""\n'
        for fk in k[0]:
            if fk not in dependclasses:
                clases[fk][3] += 1
        ordered.append([j] + k)

    i = 0
    while i < len(ordered):
        mark = i
        j = i + 1
        while j < len(ordered):
            if ordered[i][0] in ordered[j][1]:
                mark = j
            j += 1
        if mark == i:
            i += 1
        else:
            # swap ( ordered[i] , ordered[mark]) to make ordered[i] to be at the end
            if ordered[i][0] in ordered[mark][1] and ordered[mark][0] in ordered[i][1]:
                # Resolving simplistic circular ForeignKeys
                print(
                    "Not able to resolve circular ForeignKeys between %s and %s"
                    % (ordered[i][1], ordered[mark][0])
                )
                break
            a = ordered[i]
            ordered[i] = ordered[mark]
            ordered[mark] = a
        if i == len(ordered) - 1:
            break
    ordered.reverse()
    if imports:
        models_txt = str(imports)
    for i in ordered:
        models_txt += "%s\n" % str(i[3])

    return models_txt


if __name__ == "__main__":
    if len(sys.argv) == 2:
        dia2django(sys.argv[1])
    else:
        print(" Use:\n \n   " + sys.argv[0] + " diagram.dia\n\n")
